#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# %%
def thermal_conductivity(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of NbTi

    ========== VALIDITY ==========

    <temperature> : [0.1 -> 1; 4-> 9]

    ========== FROM ==========

    F. Pobell, Matter and methods at low temperatures, 3rd, rev.expanded ed
    éd. Berlin ; New York: Springer, 2007.

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <thermal_conductivity>
        -- float --
        The thermal conductivity
        [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert temperature <= 9 and temperature >= 0.1, 'The function ' \
                                                    ' Ti90Al6V4.thermal_conductivity is not defined for ' \
                                                    'T = ' + str(temperature) + ' K'

    ################## FUNCTION ###############################################

    if temperature <= 9 and temperature >= 4:
        return 0.0075 * temperature ** 1.85

    if temperature <= 1 and temperature >= 0.1:
        return 0.015 * temperature ** 2


# %%
def linear_thermal_expansion(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the linear thermal expansion of NbTi

    ========== VALIDITY ==========

    <temperature> : [4 -> 300]

    ========== FROM ==========

    E. D. Marquardt, J. P. Le, et R. Radebaugh, « Cryogenic Material
    Properties Database », p. 7, 2000.

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <linear_thermal_expansion>
        -- float --
        The linear thermal expansion
        []

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert temperature <= 300 and temperature >= 4, 'The function ' \
                                                    ' NbTi.linear_thermal_expansion is not defined for ' \
                                                    'T = ' + str(temperature) + ' K'

    ################## MODULES ################################################

    import numpy as np

    ################## INITIALISATION #########################################

    coefficients = np.array([-1.862e2, -2.568e-1, 8.334e-3, -2.951e-5, 3.908e-8])
    result = 0

    ################## FUNCTION ###############################################

    for i in range(len(coefficients)):
        result = result + coefficients[i] * temperature ** i

    return result * 1e-5



# -*- coding: utf-8 -*-
# %%
def query_resistance(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can return the measured resistance of a channel

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <channel>
        -- int --
        The channel of the instrument from 1 to 16
        
    ========== OUTPUT ==========

    <resistance>
        -- float --
        The measured resistance of the channel
        [Ohm]
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    channel = str(channel)
    answer = instru.query("RDGR? " + channel)
    resistance = float(answer)

    ################## FUNCTION ###############################################

    return resistance


# %%
def query_temperature(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can return the measured temperature of a channel

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <channel>
        -- int --
        The channel of the instrument from 1 to 16
        
    ========== OUTPUT ==========

    <temperature>
        -- float --
        The measured temperature of the channel
        [K]
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    channel = str(channel)
    answer = instru.query("RDGK? " + channel)
    temperature = float(answer)

    ################## FUNCTION ###############################################

    return temperature


# %%
def query_power(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can return the injected power on a channel

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <channel>
        -- int --
        The channel of the instrument from 1 to 16
        
    ========== OUTPUT ==========

    <power>
        -- float --
        The injected power on the channel
        [W]
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    channel = str(channel)
    answer = instru.query("RDGPWR? " + channel)
    power = float(answer)

    ################## FUNCTION ###############################################

    return power


# %%
def command_heater_range(address, value):
    """
    ========== DESCRIPTION ==========

    This function can setup the heater range of the heater output

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <value>
        -- int --
        The heater range (0 = Off
                          1 = 31.6 uA
                          2 = 100 uA
                          3 = 316 uA
                          4 = 1 mA
                          5 = 3.16 mA
                          6 = 10 mA
                          7 = 31.6 mA
                          8 = 100 mA)
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    value = str(value)

    ################## FUNCTION ###############################################

    instru.write("HTRRNG " + value)

    return


# %%
def command_heater_output(address, output):
    """
    ========== DESCRIPTION ==========

    This function can setup the heater output 

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <output>
        -- float --
        The heater output 
        [%]
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    output = str(output)

    ################## FUNCTION ###############################################

    instru.write("MOUT " + output)

    return


# %%
def query_scan(address):
    """
    ========== DESCRIPTION ==========

    This function can return the scan mode of a channel

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    ========== OUTPUT ==========

    <channel>
        -- int --
        The channel of the instrument from 1 to 16
        
    <scan_mode>
        -- int --
        The scan mode of a channel (0 = Autoscan off
                                    1 = Autoscan on)
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    answer = instru.query("SCAN?")

    channel = int(answer[0:2])
    scan_mode = int(answer[3:4])

    ################## FUNCTION ###############################################

    return channel, scan_mode


# %%
def command_scan(address, channel, scan_mode):
    """
    ========== DESCRIPTION ==========

    This function can return the scan mode of the bridge

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <channel>
        -- int --
        The channel of the instrument from 1 to 16
        
    <scan_mode>
        -- int --
        The scan mode of the bridge (0 = Autoscan off
                                     1 = Autoscan on)
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    channel = str(channel)
    scan_mode = str(scan_mode)

    ################## FUNCTION ###############################################

    instru.write("SCAN " + channel + "," + scan_mode)

    return


# %%
def command_resistance_range(
        address, channel, mode, excitation, range_value, autorange, autoexcitation
):
    """
    ========== DESCRIPTION ==========

    This function can setup the resistance reading of a channel

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <channel>
        -- int --
        The channel of the instrument from 1 to 16 (0 = All channels)

    <mode>
        -- int --
        The excitation mode (0 = voltage
                             1 = current)

    <excitation>
        -- int --
        The excitation range (1 = 2 uV      /   1 pA 
                              2 = 6.32 uV   /   3.16 pA
                              3 = 20 uV     /   10 pA
                              4 = 63.2 uV   /   31.6 pA
                              5 = 200 uV    /   100 pA
                              6 = 632 uV    /   316 pA
                              7 = 2 mV      /   1 nA
                              8 = 6.32 mV   /   3.16 nA
                              9 = 20 mV     /   10 nA
                              10 = 63.2 mV  /   31.6 nA
                              11 = 200 mV   /   100 nA
                              12 = 632 mV   /   316 nA
                              13 =          /   1 uA
                              14 =          /   3.16 uA
                              15 =          /   10 uA
                              16 =          /   31.6 uA
                              17 =          /   100 uA
                              18 =          /   316 uA
                              19 =          /   1 mA
                              20 =          /   3.16 mA
                              21 =          /   10 muA
                              22 =          /   31.6 mA)

    <range_value>
        -- int --
        The resistance range (1 = 2 mOhm
                              2 = 6.32 mOhm
                              3 = 20 mOhm
                              4 = 63.2 mOhm
                              5 = 200 mOhm
                              6 = 632 mOhm
                              7 = 2 Ohm
                              8 = 6.32 Ohm
                              9 = 20 Ohm
                              10 = 63.2 Ohm
                              11 = 200 Ohm
                              12 = 632 Ohm
                              13 = 2 kOhm
                              14 = 6.32 kOhm
                              15 = 20 kOhm
                              16 = 63.2 kOhm
                              17 = 200 kOhm
                              18 = 632 kOhm
                              19 = 2 MOhm
                              20 = 6.32 MOhm
                              21 = 20 MOhm
                              22 = 63.2 MOhm)
        
    <autorange>
        -- int --
        The autorange mode (0 = autorange off
                            1 = autorange on)

    <autoexcitation>
        -- int --
        The autoexcitation mode (0 = autoexcitation on
                                 1 = autoexcitation off)

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    channel = str(channel)
    mode = str(mode)
    excitation = str(excitation)
    range_value = str(range_value)
    autorange = str(autorange)
    autoexcitation = str(autoexcitation)

    ################## FUNCTION ###############################################

    instru.write(
        "RDGRNG "
        + channel
        + ","
        + mode
        + ","
        + excitation
        + ","
        + range_value
        + ","
        + autorange
        + ","
        + autoexcitation
    )

    return


# %%
def query_resistance_range(
        address, channel, mode, excitation, range_value, autorange, autoexcitation
):
    """
    ========== DESCRIPTION ==========

    This function can return the resistance reading of a channel

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <channel>
        -- int --
        The channel of the instrument from 1 to 16 (0 = All channels)

    ========== OUTPUT ==========
    
    <mode>
        -- int --
        The excitation mode (0 = voltage
                             1 = current)
     
    <excitation>
        -- int --
        The excitation range (1 = 2 uV      /   1 pA 
                              2 = 6.32 uV   /   3.16 pA
                              3 = 20 uV     /   10 pA
                              4 = 63.2 uV   /   31.6 pA
                              5 = 200 uV    /   100 pA
                              6 = 632 uV    /   316 pA
                              7 = 2 mV      /   1 nA
                              8 = 6.32 mV   /   3.16 nA
                              9 = 20 mV     /   10 nA
                              10 = 63.2 mV  /   31.6 nA
                              11 = 200 mV   /   100 nA
                              12 = 632 mV   /   316 nA
                              13 =          /   1 uA
                              14 =          /   3.16 uA
                              15 =          /   10 uA
                              16 =          /   31.6 uA
                              17 =          /   100 uA
                              18 =          /   316 uA
                              19 =          /   1 mA
                              20 =          /   3.16 mA
                              21 =          /   10 muA
                              22 =          /   31.6 mA)
     
    <range_value>
        -- int --
        The resistance range (1 = 2 mOhm
                              2 = 6.32 mOhm
                              3 = 20 mOhm
                              4 = 63.2 mOhm
                              5 = 200 mOhm
                              6 = 632 mOhm
                              7 = 2 Ohm
                              8 = 6.32 Ohm
                              9 = 20 Ohm
                              10 = 63.2 Ohm
                              11 = 200 Ohm
                              12 = 632 Ohm
                              13 = 2 kOhm
                              14 = 6.32 kOhm
                              15 = 20 kOhm
                              16 = 63.2 kOhm
                              17 = 200 kOhm
                              18 = 632 kOhm
                              19 = 2 MOhm
                              20 = 6.32 MOhm
                              21 = 20 MOhm
                              22 = 63.2 MOhm)
        
    <autorange>
        -- int --
        The autorange mode (0 = autorange off
                            1 = autorange on)
     
    <autoexcitation>
        -- int --
        The autoexcitation mode (0 = autoexcitation on
                                 1 = autoexcitation off)

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    channel = str(channel)

    answer = instru.query("RDGRNG? " + channel)

    mode = answer[0:1]
    excitation = answer[2:4]
    range_value = answer[5:7]
    autorange = answer[8:9]
    autoexcitation = answer[10:11]

    ################## FUNCTION ###############################################

    return mode, excitation, range_value, autorange, autoexcitation


# %%
def query_heater_range(address):
    """
    ========== DESCRIPTION ==========

    This function can return the heater range of the heater output

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    ========== OUTPUT ==========
       
    <value>
        -- int --
        The heater range (0 = Off
                          1 = 31.6 uA
                          2 = 100 uA
                          3 = 316 uA
                          4 = 1 mA
                          5 = 3.16 mA
ryopy i                          6 = 10 mA
                          7 = 31.6 mA
                          8 = 100 mA)
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    answer = instru.query("HTRRNG?")
    value = int(answer)

    ################## FUNCTION ###############################################

    return value


# %%
def query_heater_output(address):
    """
    ========== DESCRIPTION ==========

    This function can setup the heater output 

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    ========== OUTPUT ==========

    <output>
        -- float --
        The heater output 
        [%]
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    answer = instru.query("HTR?")
    output = float(answer)

    ################## FUNCTION ###############################################

    return output


# %%
def command_temperature_control_mode(address, mode):
    """
    ========== DESCRIPTION ==========

    This function can setup the temperature control mode 

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <mode>
        -- int --
        The temperature control mode (1 = Closed-loop PID
                                      2 = Zone tuning
                                      3 = Open loop
                                      4 = Off)
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    mode = str(mode)

    ################## FUNCTION ###############################################

    instru.write("CMODE " + mode)

    return


# %%
def query_temperature_control_mode(address):
    """
    ========== DESCRIPTION ==========

    This function can return the temperature control mode 

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
    
    ========== OUTPUT ==========   
    
    <mode>
        -- int --
        The temperature control mode (1 = Closed-loop PID
                                      2 = Zone tuning
                                      3 = Open loop
                                      4 = Off)
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    answer = instru.query("CMODE?")
    mode = int(answer)

    ################## FUNCTION ###############################################

    return mode


# %%
def query_channel_parameter(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can return the channel parameters

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <channel>
        -- int --
        The channel of the instrument from 1 to 16
    
    ========== OUTPUT ==========   
    
    <channel_status>
        -- int --
        The status of the channel (0 = Off
                                   1 = On)
        
    <dwell>
        -- int --
        The dwell time of the channel   
        [s]
        
    <pause>
        -- int --
        The pause time of the channel   
        [s]

    <curve_number>
        -- int --
        The curve number of the channel

    <coefficient>
        -- int --
        The temperature coefficient (1 = negative
                                     2 = positive)             
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    channel = str(channel)
    answer = instru.query("INSET? " + channel)
    channel_status = int(answer[0:1])
    dwell = int(answer[2:5])
    pause = int(answer[6:9])
    curve_number = int(answer[10:12])
    coefficient = int(answer[13:14])

    ################## FUNCTION ###############################################

    return channel_status, dwell, pause, curve_number, coefficient


# %%
def command_channel_parameter(
        address, channel, channel_status, dwell, pause, curve_number, coefficient
):
    """
    ========== DESCRIPTION ==========

    This function can setup the channel parameters

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <channel>
        -- int --
        The channel of the instrument from 1 to 16 
    
    <channel_status>
        -- int --
        The status of the channel (0 = Off
                                   1 = On)
        
    <dwell>
        -- int --
        The dwell time of the channel   
        [s]
        
    <pause>
        -- int --
        The pause time of the channel   
        [s]

    <curve_number>
        -- int --
        The curve number of the channel

    <coefficient>
        -- int --
        The temperature coefficient (1 = negative
                                     2 = positive)             
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """
    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    channel = str(channel)
    channel_status = str(channel_status)
    dwell = str(dwell)
    pause = str(pause)
    curve_number = str(curve_number)
    coefficient = str(coefficient)

    ################## FUNCTION ###############################################

    instru.write(
        "INSET "
        + channel
        + ","
        + channel_status
        + ","
        + dwell
        + ","
        + pause
        + ","
        + curve_number
        + ","
        + coefficient
    )
    return


# %%
def command_interface_mode(address, interface_mode):
    """
    ========== DESCRIPTION ==========

    This function can setup the interface mode

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <interface_mode>
        -- int --
        The interface mode (0 = local
                            1 = remote
                            2 = remote with local lockout)             
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """
    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    interface_mode = str(interface_mode)

    ################## FUNCTION ###############################################

    instru.write("MODE " + interface_mode)
    return


# %%
def query_interface_mode(address):
    """
    ========== DESCRIPTION ==========

    This function can setup the interface mode

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    ========== OUTPUT ==========
    
    <interface_mode>
        -- int --
        The interface mode (0 = local
                            1 = remote
                            2 = remote with local lockout)             
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """
    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    answer = instru.query("MODE?")
    interface_mode = int(answer)

    ################## FUNCTION ###############################################

    return interface_mode


# %%
def query_pid_parameters(address):
    """
    ========== DESCRIPTION ==========

    This function can return the pid parameters

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    ========== OUTPUT ==========
    
    <p>
        -- float --
        The "p" parameter
    []

    <i>
        -- int --
        The "i" parameter
    [s]

    <d>
        -- int --
        The "d" parameter
    [s]
         
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """
    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    answer = instru.query("MODE?")
    p = float(answer[0:7])
    i = int(answer[9:15])
    d = int(answer[17:23])

    ################## FUNCTION ###############################################

    return p, i, d


# %%
def command_pid_parameters(address, p, i, d):
    """
    ========== DESCRIPTION ==========

    This function can setup the pid parameters

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
    
    <p>
        -- float --
        The "p" parameter
    []

    <i>
        -- int --
        The "i" parameter
    [s]

    <d>
        -- int --
        The "d" parameter
    [s]
         
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """
    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    p = str(p)
    i = str(i)
    d = str(d)

    ################## FUNCTION ###############################################

    instru.write("PID " + p + "," + i + "," + d)

    return


# %%
def query_setpoint(address):
    """
    ========== DESCRIPTION ==========

    This function can return the setpoint

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
     
    ========== OUTPUT ==========

    <setpoint>
        -- float --
        The setpoint 
        [K] or [Ohm]
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """
    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    answer = instru.query("SETP?")
    setpoint = float(answer)

    ################## FUNCTION ###############################################

    return setpoint


# %%
def command_setpoint(address, setpoint):
    """
    ========== DESCRIPTION ==========

    This function can setup the setpoint

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
     
    <setpoint>
        -- float --
        The setpoint 
        [K] or [Ohm]
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """
    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    setpoint = str(setpoint)

    ################## FUNCTION ###############################################

    instru.write("SETP" + setpoint)
    return


# %%
def preview_heater_range(resistance, power):
    """
    ========== DESCRIPTION ==========

    This function can return the adapted heater range value 

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========
    
    <resistance>
        -- float --
        The resistance value of the heater
        [Ohm]

    <power>
        -- float --
        The maximum expected power value of the heater
        [Ohm]
        
    ========== OUTPUT ==========

    <range_value>
        -- int --
        The heater range value
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    ################## INITIALISATION #########################################

    range_intensity = [0, 31.6e-6, 100e-6, 316e-6, 1e-3, 3.16e-3, 10e-3, 31.6e-3, 0.1]
    power_value = [resistance * i ** 2 for i in range_intensity]

    for i in range(len(power_value)):
        if power_value[i] > power:
            range_value = i - 1

    ################## FUNCTION ###############################################

    return range_value


# %%
def command_temperature_control_parameters(
        address, channel, filtering, unit, delay, current_power, htr_limit, htr_resistance
):
    """
    ========== DESCRIPTION ==========

    This function can setup the contrôl temperature parameters

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <channel>
        -- int --
        The channel of the instrument to follow from 1 to 16 
    
    <filtering>
        -- int --
        The status of the filter (0 = unfiltered
                                  1 = filtered)
        
    <unit>
        -- int --
        The setpoint unit (1 = Kelvin
                           2 = Ohm)   
        
    <delay>
        -- int --
        Delay in seconds for setpoint change during Autoscanning: 1–255
        [s]

    <current_power>
        -- int --
        Specifies heater output display in current or power (1 = current
                                                             2 = power)

    <htr_limit>
        -- int --
        Maximum heater range from 1 to 8

    <htr_resistance>
        -- int --
        Heater load in ohms
        [Ohm]
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """
    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    channel = str(channel)
    filtering = str(filtering)
    unit = str(unit)
    delay = str(delay)
    current_power = str(current_power)
    htr_limit = str(htr_limit)
    htr_resistance = str(htr_resistance)

    ################## FUNCTION ###############################################

    instru.write(
        "CSET "
        + channel
        + ","
        + filtering
        + ","
        + unit
        + ","
        + delay
        + ","
        + current_power
        + ","
        + htr_limit
        + ","
        + htr_resistance
    )
    return


# %%
def query_temperature_control_parameters(address):
    """
    ========== DESCRIPTION ==========

    This function can return the control temperature parameters

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    <channel>
        -- int --
        The channel of the instrument to follow from 1 to 16 
    
    <filtering>
        -- int --
        The status of the filter (0 = unfiltered
                                  1 = filtered)
        
    <unit>
        -- int --
        The setpoint unit (1 = Kelvin
                           2 = Ohm)   
        
    <delay>
        -- int --
        Delay in seconds for setpoint change during Autoscanning: 1–255
        [s]

    <current_power>
        -- int --
        Specifies heater output display in current or power (1 = current
                                                             2 = power)

    <htr_limit>
        -- int --
        Maximum heater range from 1 to 8

    <htr_resistance>
        -- int --
        Heater load in ohms
        [Ohm]
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """
    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    answer = instru.query("CSET?")
    channel = int(answer[0:2])
    filtering = int(answer[3:4])
    unit = int(answer[5:6])
    delay = int(answer[7:10])
    current_power = int(answer[11:12])
    htr_limit = int(answer[13:14])
    htr_resistance = int(answer[15:23])

    ################## FUNCTION ###############################################

    return channel, filtering, unit, delay, current_power, htr_limit, htr_resistance


# %%
def query_intensity_output(address):
    """
    ========== DESCRIPTION ==========

    This function can return the intensity output

    ========== FROM ==========

    Manual of Lakeshore 370 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    ========== OUTPUT ==========

    <intensity_output>
        -- float --
        The intensity output 
        [A]
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """
    ################## MODULES ################################################

    from cryopy.Instrument import Lakeshore370

    ################## INITIALISATION #########################################

    heater_range_value = [
        0,
        31.6e-6,
        100e-6,
        316e-6,
        1e-3,
        3.16e-3,
        10e-3,
        31.6e-3,
        100e-3,
    ]
    heater_range = heater_range_value[Lakeshore370.query_heater_range(address)]
    heater_percentage = Lakeshore370.query_heater_output(address)

    ################## FUNCTION ###############################################

    return heater_range * heater_percentage



def temperature_function(temperature, pressure, fraction_3he):
    """
    ========== DESCRIPTION ==========

    This function return the Fermi temperature of the same density
    quasi-particule of Helium 3

    ========== VALIDITY ==========

    <pressure> : ...
    <fraction_3he> : [0 -> 0.08]

    ========== FROM ==========

    Kuerten et al. - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Equation (7)

    ========== INPUT ==========

    <pressure>
        -- float --
        The pressure
        [Pa]

    <fraction_3he>
        -- float --
        The fraction of Helium 3 on mixture
        []

    ========== OUTPUT ==========

    <temperature>
        -- float --
        The Fermi temperature
        [J].[K]**(-1).[mol]**(-1)


    ========== STATUS ==========

    Status : Checked

    ========== NOTES ===========

    """

    ################## MODULES ################################################

    from cryopy.Helium import Helium3, Helium7
    from cryopy import Constant
    import numpy as np

    ################## FUNCTION ###############################################

    return Constant.reduced_planck() ** 2 / (2 * Helium3.effective_mass() * Constant.boltzmann()) * (
            3 * np.pi ** 2 * Constant.avogadro() * fraction_3he / Helium7.molar_volume(temperature, pressure,
                                                                                       fraction_3he)) ** (2 / 3)


def molar_specific_heat(temperature, pressure, fraction_3he):
    """
    ========== DESCRIPTION ==========

    This function return the molar specific heat of an ideal Fermi gas with
    the same density of quasiparticules as the mixture with the effective mass
    of helium 3

    ========== VALIDITY ==========

    <temperature> : [0 -> 0.250]
    <pressure> : ...
    <fraction_3he> : [0 -> 0.08]

    ========== SOURCE ==========

    Kuerten at al. - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Equation (42)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the mixture
        [K]

    <pressure>
        -- float --
        The pressure
        [Pa]

    <fraction_3he>
        -- float --
        The fraction of Helium 3 on mixture
        []

    ========== OUTPUT ==========

    <molar_specific_heat>
        -- float --
        The molar specific heat of the Fermi ideal gas
        [J].[K]**(-1).[mol]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ################################################

    from cryopy import Constant
    from cryopy.Helium import Fermi
    import numpy as np
    import pandas as pd

    ################## CONDITIONS #############################################

    assert 0.08 >= fraction_3he >= 0, 'The function ' \
                                      ' Fermi.molar_specific_heat is not defined for ' \
                                      'x = ' + str(fraction_3he * 100) + ' %'

    assert 0.250 >= temperature >= 0, 'The function ' \
                                      ' Fermi.molar_specific_heat is not defined for ' \
                                      'T = ' + str(temperature) + ' K'

    ################## INITIALISATION #########################################

    result = 0
    t = temperature / Fermi.temperature_function(temperature, pressure, fraction_3he)
    coefficients = pd.DataFrame(np.array([[4.934802, -0.201414, 1.5],
                                          [-14.400636, 8.910152, -0.09973557],
                                          [-167.8453, -27.147564, 0.00560236],
                                          [-4313.1735, 56.254979, -0.00024872],
                                          [203138.64, -77.719456, np.nan],
                                          [np.nan, 62.61363, np.nan],
                                          [np.nan, -21.64979, np.nan]]),
                                columns=['A_1', 'A_2', 'A_3'])

    ################## FUNCTION ###############################################

    if t <= 0.15:
        for j in [0, 1, 2, 3, 4]:
            result = result + coefficients.A_1[j] * t ** (2 * j + 1)
        return Constant.gas() * result
    if 0.7 >= t >= 0.15:
        for j in [0, 1, 2, 3, 4, 5, 6]:
            result = result + coefficients.A_2[j] * t ** j
        return Constant.gas() * result
    if t >= 0.7:
        for j in [0, 1, 2, 3]:
            result = result + coefficients.A_3[j] * t ** (-3 * j / 2)
        return Constant.gas() * result


def molar_entropy(temperature, pressure, fraction_3he):
    """
    ========== DESCRIPTION ==========

    This function return the molar entropy of an ideal Fermi gas with
    the same density of quasiparticules as the mixture with the effective mass
    of Helium 3

     ========== VALIDITY ==========

     <temperature> : [0 -> 0.250]
     <pressure> : ...
     <fraction_3he> : [0 -> 0.08]

     ========== SOURCE ==========

     Kuerten at al. - Thermodynamic properties of liquid 3He-4He mixtures
     at zero pressure for temperatures below 250 mK and 3He concentrations
     below 8% - Equation (8)

     ========== INPUT ==========

     <temperature>
         -- float --
         The temperature of the mixture
         [K]

    <pressure>
        -- float --
        The pressure
        [Pa]

     <fraction_3he>
         -- float --
         The fraction of Helium 3 on mixture
         []

     ========== OUTPUT ==========

     <molar_entropy>
         -- float --
         The molar entropy of the Fermi ideal gas
         [J].[K]**(-1).[mol]**(-1)

     ========== STATUS ==========

     Status : checked

    """

    ################## MODULES ###############################################

    from cryopy import Constant
    from cryopy.Helium import Fermi
    import numpy as np
    import pandas as pd

    ################## CONDITIONS #############################################

    assert 0.08 >= fraction_3he >= 0, 'The function Fermi.molar_entropy is not defined for x = ' + str(
        fraction_3he * 100) + ' %'
    assert 0.250 >= temperature >= 0, "The function Fermi.molar_entropy is not defined for T = " + str(
        temperature) + ' K'

    ################## INITIALISATION #########################################

    result = 0
    t = temperature / Fermi.temperature_function(temperature,pressure, fraction_3he)
    coefficients = pd.DataFrame(np.array([[4.934802, -0.201414, 1.5],
                                          [-14.400636, 8.910152, -0.09973557],
                                          [-167.8453, -27.147564, 0.00560236],
                                          [-4313.1735, 56.254979, -0.00024872],
                                          [203138.64, -77.719456, np.nan],
                                          [np.nan, 62.61363, np.nan],
                                          [np.nan, -21.64979, np.nan]]),
                                columns=['A_1', 'A_2', 'A_3'])

    ################## FUNCTION ###############################################

    if t <= 0.15:
        for j in [0, 1, 2, 3, 4]:
            result = result + coefficients.A_1[j] * t ** (2 * j + 1) / (2 * j + 1)
        return Constant.gas() * result

    if 0.7 >= t > 0.15:
        result = result + coefficients.A_2[0] * np.log(t) - 0.7462941987437959
        for j in [1, 2, 3, 4, 5, 6]:
            result = result + coefficients.A_2[j] * (t ** j) / j
        return Constant.gas() * result

    if t > 0.7:
        result = result + coefficients.A_3[0] * np.log(t) + 2.779744103011612 - 2 / 3 * coefficients.A_3[1] * t ** (
                -3 / 2) - 1 / 3 * coefficients.A_3[2] * t ** (-3) - 9 / 2 * coefficients.A_3[3] * t ** (-9 / 2)
        return Constant.gas() * result


def chemical_potential(temperature, pressure, fraction_3he):
    """
    ========== DESCRIPTION ==========

    This function return the molar entropy of an ideal Fermi gas with
    the same density of quasiparticules as the mixture with the effective mass
    of Helium 3

     ========== VALIDITY ==========

    <temperature> : ...
    <pressure> : ...
    <fraction_3he> : ...

     ========== FROM ==========

     Kuerten at al. - Thermodynamic properties of liquid 3He-4He mixtures
     at zero pressure for temperatures below 250 mK and 3He concentrations
     below 8% - Equation (15)

     ========== INPUT ==========

     <temperature>
         -- float --
         The temperature of the mixture
         [K]

    <pressure>
        -- float --
        The pressure
        [Pa]

     <fraction_3he>
         -- float --
         The fraction of Helium 3 on mixture
         []

     ========== OUTPUT ==========

     <chemical_potential>
         -- float --
         The chemical potential of a Fermi gas
         [J].[mol]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ################################################

    from cryopy import Constant
    from cryopy.Helium import Fermi
    from scipy import integrate

    ################## INITIALISATION #########################################

    t = temperature / Fermi.temperature_function(temperature,pressure, fraction_3he)

    ################## FUNCTION ###############################################

    return Constant.gas() * Fermi.temperature_function(temperature, pressure, fraction_3he) + 5 / 3 * Fermi.temperature_function(temperature,
        pressure, fraction_3he) * integrate.quad(
        Fermi.molar_specific_heat, 0, t, args=(pressure, fraction_3he)) - temperature * Fermi.molar_entropy(temperature,
                                                                                                            pressure,
                                                                                                            fraction_3he)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# %%
def thermal_conductivity(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of Cu80Ni20

    ========== VALIDITY ==========

    <temperature> : [2 -> 20]

    ========== FROM ==========


    J. K. Hulm, « The Thermal Conductivity of a Copper-Nickel Alloy at
    Low Temperatures », Proc. Phys. Soc. B, vol. 64, no 3, p. 207‑211,
    mars 1951, doi: 10.1088/0370-1301/64/3/304.

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <thermal_conductivity>
        -- float --
        The thermal conductivity
        [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert temperature <= 20 and temperature >= 2, 'The function ' \
                                                   ' Cu80Ni20.thermal_conductivity is not defined for ' \
                                                   'T = ' + str(temperature) + ' K'

    ################## FUNCTION ###############################################

    result = 2.2 * 1e-2 * temperature ** 2

    return result



#%%
def molar_mass():
    """
    ========== DESCRIPTION ==========

    Return the constant molar mass of a single helium 3 atom.

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    P. J. Mohr, B. N. Taylor, and D. B. Newell, Rev. Mod. Phys. 84, 1531 (2012).

    ========== OUTPUT ==========

    <molar_mass>
        -- float --
    	The molar mass of a single atom of helium 3
        [kg].[mol]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## RETURN #################################################

    return 3.0160293191e-3

#%%
def mass():
    """
    ========== DESCRIPTION ==========

    Return the constant mass of a single helium 3 atom.

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    P. J. Mohr, B. N. Taylor, and D. B. Newell, Rev. Mod. Phys. 84, 1531 (2012).

    ========== OUTPUT ==========

    <mass>
        -- float --
    	The mass of a single atom of helium 3
        [kg]

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ################################################

    from cryopy import constant
    from cryopy.helium import helium3

    ################## RETURN #################################################

    return helium3.molar_mass() / constant.avogadro()

#%%
def effective_mass():
    """
    ========== DESCRIPTION ==========

    Return the effective mass of a single atom of helium 3.
    This function does not require parameter since this is a constant

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    KUERTEN - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Equation (44)

    POBELL - Matter and methods at low temperature - Second edition -  P.125

    ========== OUTPUT ==========

    <effective_mass>
        -- float --
    	The effective mass of a single atom of helium 3
        [kg]

    ========== STATUS ==========

    Status : Checked

    ========== NOTES ===========

    A potential improvement can be done since the effective mass of helium 3
    can change if helium 3 is in a helium 3 /helium 4 mixture

    """

    ################## MODULES ################################################

    from cryopy.helium import helium3

    ################## RETURN ###############################################

    return 2.46 * helium3.mass()

#%%
def molar_specific_heat(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the molar specific heat of pure helium 3

    ========== VALIDITY ==========

    <temperature> : [0 -> 1.8]
    <pressure> : [0]

    ========== FROM ==========

    KUERTEN - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Equation (48)

    CHAUDHRY - Thermodynamic properties of liquid 3He-4he mixtures
    between 0.15 K and 1.8 K - Equation (A14)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium 3
        [K]

    <pressure>
        -- float --
        The pressure of helium 3
        [Pa]

    ========== OUTPUT ==========

    <molar_specific_heat>
        -- float --
        The molar specific heat of pure helium 3
        [J].[K]**(-1).[mol]**(-1)


    ========== STATUS ==========

    Status : Checked

    ========== NOTES ===========

    This function requires the implementation of pressure changes


    """

    ################## CONDITIONS #############################################

    assert pressure <= 0 and pressure >= 0, 'The function ' \
                                            ' helium3.molar_specific_heat is not defined for ' \
                                            'P = ' + str(pressure) + ' Pa'

    assert temperature <= 1.8 and temperature >= 0.000, 'The function ' \
                                                        ' helium3.molar_specific_heat is not defined for ' \
                                                        'T = ' + str(temperature) + ' K'

    ################## MODULES ################################################

    from cryopy import constant
    import pandas as pd
    import numpy as np

    ################## INITIALISATION #########################################

    result = 0
    Coefficients = pd.DataFrame(np.array([[np.nan, 0.0245435],
                                          [2.7415, 1.85688],
                                          [0, 9.39988],
                                          [-61.78929, -117.910],
                                          [-177.8937, 440.368],
                                          [2890.0675, -735.836],
                                          [0, 468.741]]),
                                columns=['C_1', 'C_2'])

    ################## FUNCTION ###############################################

    if temperature <= 0.1:
        for j in [1, 2, 3, 4, 5]:
            result = result + Coefficients.C_1[j] * temperature ** j
        return constant.gas() * result

    if temperature <= 0.45 and temperature > 0.1:
        for j in [0, 1, 2, 3, 4, 5, 6]:
            result = result + Coefficients.C_2[j] * temperature ** j
        return constant.gas() * result

    else:
        return 3.6851551 - 1.9650072 * temperature + 3.3601049 * temperature ** 2 - 0.8351251 * temperature ** 3 - (
                0.0444842 / (temperature ** 2)) * np.exp(-0.0977175 / temperature)

#%%
def molar_enthalpy(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function can return the molar enthalpy of helium 3

    ========== VALIDITY ==========

    <temperature> : [0 -> 0.450]
    <pressure> : [0]

    ========== FROM ==========

    KUERTEN - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Equation (37)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium 3
        [K]

    <pressure>
        -- float --
        The pressure of helium 3
        [Pa]

    ========== OUTPUT ==========

    <molar_enthalpy>
        -- float --
        The molar enthalpy of pure helium 3
        [J].[mol]**(-1)


    ========== STATUS ==========

    Status : Checked

    ========== NOTES ===========

    This function requires the implementation of pressure changes

    """

    ################## CONDITIONS #############################################

    assert pressure <= 0 and pressure >= 0, 'The function ' \
                                            ' helium3.molar_enthalpy is not defined for ' \
                                            'P = ' + str(pressure) + ' Pa'

    assert temperature <= 0.450 and temperature >= 0.000, 'The function ' \
                                                          ' helium3.molar_enthalpy is not defined for ' \
                                                          'T = ' + str(temperature) + ' K'

    ################## MODULES ################################################

    from cryopy import constant
    import pandas as pd
    import numpy as np

    ################## INITIALISATION #########################################

    result = 0
    Coefficients = pd.DataFrame(np.array([[np.nan, 0.0245435],
                                          [2.7415, 1.85688],
                                          [0, 9.39988],
                                          [-61.78929, -117.910],
                                          [-177.8937, 440.368],
                                          [2890.0675, -735.836],
                                          [0, 468.741]]),
                                columns=['C_1', 'C_2'])

    ################## FUNCTION ###############################################

    if temperature <= 0.1:
        for j in [1, 2, 3, 4, 5]:
            result = result + Coefficients.C_1[j] * temperature ** (j + 1) / (j + 1)
        return constant.gas() * result

    if temperature <= 0.45 and temperature > 0.1:
        result = result - 0.0004006515045376974
        for j in [0, 1, 2, 3, 4, 5, 6]:
            result = result + Coefficients.C_2[j] * temperature ** (j + 1) / (j + 1)
        return constant.gas() * result

#%%
def molar_entropy(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function can return the molar entropy of helium 3

    ========== VALIDITY ==========

    <temperature> : [0 -> 0.450]
    <pressure> : [0]

    ========== FROM ==========

    KUERTEN - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Equation (37)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium 3
        [K]

    <pressure>
        -- float --
        The pressure of helium 3
        [Pa]

    ========== OUTPUT ==========

    <molar_entropy>
        -- float --
        The molar entropy of pure helium 3
        [J].[K]**(-1).[mol]**(-1)


    ========== STATUS ==========

    Status : Checked

    ========== NOTES ===========

    This function requires the implementation of pressure changes


    """

    ################## MODULES ################################################

    import numpy as np

    ################## CONDITIONS #############################################

    assert pressure <= 0 and pressure >= 0, 'The function ' \
                                            ' helium3.molar_entropy is not defined for ' \
                                            'P = ' + str(pressure) + ' Pa'

    assert temperature <= 0.450 and temperature >= 0.000, 'The function ' \
                                                          ' helium3.molar_entropy is not defined for ' \
                                                          'T = ' + str(temperature) + ' K'

    ################## INITIALISATION #########################################

    Coefficients = [5.64822229e+02, -8.94486733e+02, 5.54757487e+02,
                    -1.45763639e+02, -9.26752184e+00, 2.31209308e+01,
                    -1.62015854e-03]
    Polynome = np.poly1d(Coefficients)

    ################## FUNCTION ###############################################

    return Polynome(temperature)

#%%
def chemical_potential(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the chemical potential of helium 3

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    KUERTEN - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Equation (38)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium 3
        [K]

    <pressure>
        -- float --
        The pressure of helium 3
        [Pa]

    ========== OUTPUT ==========

    <chemical_potential>
        -- float --
        The chemical potential of pure helium 3
        [J].[mol]**(-1)

    ========== STATUS ==========

    Status : In progress (need to be verified with new values of pressure)

    """

    ################## MODULES ################################################

    from cryopy.helium import helium3

    ################## RETURN #################################################

    return helium3.molar_enthalpy(temperature, pressure) - temperature * helium3.molar_entropy(temperature, pressure)


#%%
def internal_energy(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the internal energy of helium 3

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    KUERTEN - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Equation (37)

    ========== INPUT ==========

    <temperature>
        The temperature of helium 3
        [K]

    <pressure>
        The pressure of helium 3
        [Pa]

    ========== OUTPUT ==========

    <internal_energy>
        The internal energy of pure helium 3
        [J].[mol]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : In progress (need to be verified with new values of pressure)

    """

    ################## MODULES ################################################

    from cryopy.helium import helium3

    ################## FUNCTION ###############################################

    return helium3.molar_enthalpy(temperature, pressure)

#%%
def molar_volume(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the molar volume of a mole of helium 3

    ========== VALIDITY ==========

    <temperature> : [0 -> 1]
    <pressure> : [0 -> 15.7e5]


    ========== FROM ==========

    CHAUDHRY - Thermodynamic properties of liquid 3He-4he mixtures
    between 0.15 K and 1.8 K - Equation (A.22)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium 3
        [K]

    <pressure>
        -- float --
        The pressure of helium 3
        [Pa]

    ========== OUTPUT ==========

    <molar_volume>
        -- float --
        The molar volume of helium 3
        [m]**3.[mol]**(-1)

    ========== STATUS ==========

    Status : Checked


    """

    ################## MODULES ################################################

    import numpy as np

    ################## CONDITIONS #############################################

    assert 0 <= pressure <= 15.7e5 , 'The function helium3.molar_volume is not defined for P = ' + str(pressure) + ' Pa'
    assert 0 <= temperature <= 1, 'The function helium3.molar_volume is not defined for T = ' + str(temperature) + ' K'

    ################## INITIALISATION #########################################

    Coefficients = np.array([[40.723012, -1.3151948, 0.0409498],
                             [-0.6614794, 0.1275125, -0.0091931],
                             [0.5542147, -0.1527959, 0.0113764],
                             [0.1430724, -0.0034712, -0.0008515],
                             [-0.2603492, np.nan, -0.0051946]])

    pressure = pressure * 1e-5  # From [Pa] to [Bar]

    result = 0

    ################## FUNCTION ###############################################

    for i in range(3):
        for j in range(2):
            result = result + Coefficients[i][j] * temperature ** i * pressure ** j

    return (result + 1 / (Coefficients[4][2] * pressure ** 2 + Coefficients[4][0])) * 1e-6  # 1e-6 to SI

#%%
def density(temperature, pressure):
    """
    ========== DESCRIPTION ==========
    
    This function return the density of pure helium 3

    ========== VALIDITY ==========

    ========== FROM ==========

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium 3
        [K]
        
    <pressure>
        -- float --
        The pressure of helium 3
        [Pa]
        
    ========== OUTPUT ==========
    
    <density>
        -- float --
        The density of helium 3
        [kg].[m]**3
    
    ========== STATUS ==========     
    
    Status : Checked

    """

    ################## CONDITIONS #############################################

    ################## INITIALISATION #########################################

    ################## FUNCTION ###############################################

    return molar_mass()/molar_volume(temperature,pressure)

#%%
def solid_liquid_transition(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the pressure of the Solid/Liquid transition of helium 3 at a given temperature

    ========== VALIDITY ==========

    <temperature> : [1 -> 3.15]

    ========== FROM ==========

    SHERMAN - pressure-Volume-temperature Relations of Liquid He3 from 1K
    to 3.3K - Equation (1)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium 3
        [K]

    ========== OUTPUT ==========

    <solid_liquid_transition>
        -- float --
        The pressure of solid/liquid helium 3 transition
        [Pa]

    ========== STATUS ==========

    Status : Checked


    """
    ################## CONDITIONS #############################################

    assert temperature <= 3.15 and temperature >= 1, 'The function ' \
                                                     ' helium3.molar_volume is not defined for ' \
                                                     'T = ' + str(temperature) + ' K'

    ################## FUNCTION ###############################################

    return (24.599 + 16.639 * temperature ** 2 - 2.0659 * temperature ** 3 + 0.11212 * temperature ** 4) * 1e5

#%%
def liquid_thermal_conductivity(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of helium 3

    ========== VALIDITY ==========

    <temperature> : [0.003 -> 300]
    <pressure> : [0 -> 20e6]

    ========== FROM ==========

    HUANG - Thermal conductivity of helium-3 between 3 mK and 300 K

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium 3
        [K]

    <pressure>
        -- float --
        The pressure of helium 3
        [Pa]

    ========== OUTPUT ==========

    <liquid_thermal_conductivity>
        -- float --
        The thermal conductivity of liquid helium 3
        [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked


    """

    ################## MODULES ################################################

    import numpy as np

    ################## CONDITIONS #############################################

    assert pressure <= 20e6 and pressure >= 0, 'The function ' \
                                               ' helium3.liquid_thermal_conductivity is not defined for ' \
                                               'P = ' + str(pressure) + ' Pa'

    assert temperature <= 300 and temperature >= 0.003, 'The function ' \
                                                        ' helium3.liquid_thermal_conductivity is not defined for ' \
                                                        'T = ' + str(temperature) + ' K'

    ################## INITIALISATION #########################################

    Coefficients = np.array([[-4.96046174, -0.65469605, 0.22709943, -0.048878330, -0.014822690, -0.0065179072],
                             [0.73147012, 1.1285816, -0.35962597, 0.068654087, 0.016518400, np.nan],
                             [0.89421626, -0.079608445, 0.018861074, 0.029884542, np.nan, np.nan],
                             [-0.0084984819, -0.054117908, 0.016515130, np.nan, np.nan, np.nan],
                             [-0.10296191, -0.022615875, np.nan, np.nan, np.nan, np.nan],
                             [-0.015211058, np.nan, np.nan, np.nan, np.nan, np.nan]])

    x = np.log(temperature)  # Log scale of the temperature
    y = pressure / 101325  # From [Pa] to [Bar]

    x = x * 2 / np.log(300 / 0.003) - np.log(0.003) * 2 / np.log(300 / 0.003) - 1  # Normalize between -1 and 1
    y = y * 0.010132500000000001 - 1  # Normalize between -1 and 1

    ################## FUNCTION ###############################################

    result = 0

    for j in np.arange(1, 6):
        for i in np.arange(1, 6 - j):
            result = result + Coefficients[i][j] * np.cos(i * np.arccos(x)) * np.cos(j * np.arccos(y))

    for i in np.arange(1, 6):
        result = result + Coefficients[i][0] * np.cos(i * np.arccos(x))

    for j in np.arange(1, 6):
        result = result + Coefficients[0][j] * np.cos(j * np.arccos(y))

    result = result + Coefficients[0][0]

    return np.exp(result)

#%%
def gas_thermal_conductivity(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of gaseous helium 3

    ========== VALIDITY ==========

    <temperature> : [0.003 -> 300]
    <pressure> : [0 -> 20e6]

    ========== FROM ==========

    HUANG - Thermal conductivity of helium-3 between 3 mK and 300 K

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium 3
        [K]

    <pressure>
        -- float --
        The pressure of helium 3
        [Pa]

    ========== OUTPUT ==========

    <thermal_conductivity_gas>
        -- float --
        The thermal conductivity of gaseous helium 3
        [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : 

    """

    ################## CONDITIONS #############################################

    assert pressure <= 20e6 and pressure >= 0, 'The function ' \
                                               ' helium3.gas_thermal_conductivity is not defined for ' \
                                               'P = ' + str(pressure) + ' Pa'

    assert temperature <= 300 and temperature >= 0.003, 'The function ' \
                                                        ' helium3.gas_thermal_conductivity is not defined for ' \
                                                        'T = ' + str(temperature) + ' K'

    ################## MODULES ################################################

    import numpy as np

    ################## INITIALISATION #########################################

    Coefficients = np.array([[85.487315, 5.495659, 3.089753, 1.276371, 0.577591],
                             [98.529019, -4.351606, -2.525931, -0.430895, np.nan],
                             [38.396887, -1.862116, -0.798918, np.nan, np.nan],
                             [12.020784, 2.80104, np.nan, np.nan, np.nan],
                             [2.159515, np.nan, np.nan, np.nan, np.nan]])

    # Log scale of the temperature
    x = np.log(temperature)

    # From [Pa] to [Bar]
    y = pressure / 101325

    # Normalize between -1 and 1
    x = x * 2 / np.log(300 / 0.003) - np.log(0.003) * 2 / np.log(300 / 0.003) - 1

    # Normalize between -1 and 1
    y = y * 0.010132500000000001 - 1

    ################## FUNCTION ###############################################

    result = 0

    for j in np.arange(1, 5):
        for i in np.arange(1, 5 - j):
            result = result + Coefficients[i][j] * np.cos(i * np.arccos(x)) * np.cos(j * np.arccos(y))

    for i in np.arange(1, 5):
        result = result + Coefficients[i][0] * np.cos(i * np.arccos(x))

    for j in np.arange(1, 5):
        result = result + Coefficients[0][j] * np.cos(j * np.arccos(y))

    result = result + Coefficients[0][0]

    return np.exp(result)

#%%
def saturated_vapour_pressure(temperature):

    """
    ========== DESCRIPTION ==========

    This function return the saturated vapour pressure of helium 3

    ========== FROM ==========

    Huang 2006

    ========== INPUT ==========

    <temperature>
            -- float --
            The temperature of helium 3
            [K]

    ========== OUTPUT ==========

    <saturated_vapour_pressure>
            -- float --
            The saturated vapour pressure
            [Pa]

    ========== STATUS ==========

    Status : Checked

    """
    ################## MODULES ################################################

    import numpy as np

    ################## CONDITIONS #############################################

    assert 0.2 <= temperature <= 3.3162, 'The function  helium3.saturated_vapour_pressure is not defined for T = ' + str(temperature) + ' K'

    ################## INITIALISATION #########################################

    result = 0
    a = [-2.50943,9.70876,-0.304433,0.210429,-0.0545145,0.0056067]
    b = 2.25484
    c = 3.3162

    ################## FUNCTION ###############################################

    for i in range(len(a)):
        result = result + a[i]*temperature**(i-1)

    result = result + b*np.log(temperature)
    result = np.exp(result)

    return result

#%%
def dynamic_viscosity(temperature,pressure):

    """
    ========== DESCRIPTION ==========

    This function return the dynamic viscosity of pure liquid helium 3

    ========== FROM ==========

    Huang et al. (2011)
        "Viscosity of liquid and gaseous helium-3 from 3 mK to 500 K"

    ========== INPUT ==========

    <temperature>
            -- float --
            The temperature of pure helium 3
            [K]

    <pressure>
            -- float --
            The pressure of pure helium 3
            [Pa]

    ========== OUTPUT ==========

    <liquid_dynamic_viscosity>
            -- float --
            The dynamic viscosity of pure liquid helium 3
            [Pa.s]

    ========== STATUS ==========

    Status : 

    """
    ################## MODULES ################################################

    import numpy as np
    from numpy.polynomial.chebyshev import chebval2d

    ################## CONDITIONS #############################################

    assert temperature <= 3.3157 and temperature >= 0.003, 'The function helium3.dynamic_viscosity is not defined for T = ' + str(temperature) + ' K'
    assert pressure <= 20e6 and pressure >= 0, 'The function helium3.dynamic_viscosity is not defined for pressure = ' + str(pressure) + ' Pa'

    ################## INITIALISATION #########################################

    coefficients = np.array([[-8.502104326,-0.212634991,-0.1284320028,-0.0348041443,-0.0109054008],
                             [-5.781126976,0.364977795,0.1265706577,0.0968821863,0],
                             [1.0382238703,0.0208418086,0.0685042034,0,0],
                             [0.3584021376,0.0238452264,0,0,0],
                             [-0.0386854588,0,0,0,0]])
    
    # temperature scaling
    
    temperature = ((np.log(0.001)-np.log(temperature))/(np.log(0.001)-np.log(3.3157)))*2-1

    # pressure scaling
    
    pressure = ((pressure/101325)/(20e6/101325))*2-1

    ################## FUNCTION ###############################################

    return np.exp(chebval2d(temperature,pressure,coefficients))



#%% sandbox


def viscous_heating(temperature,pressure):
    import numpy as np
    flowrate = 30e-6
    diameter = 200e-6
    return 128*dynamic_viscosity(temperature,pressure)*flowrate**2*molar_volume(temperature,pressure)/(np.pi*diameter)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# %%
def ThermalConductivity(Temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of Ni60Cr15Fe16Mo7
    
    ========== Validity ==========

    2K < Temperature < 20K

    ========== FROM ==========
    
    J. K. Hulm, « The Thermal Conductivity of a Copper-Nickel Alloy at Low Temperatures », Proc. Phys. Soc. B, vol. 64, no 3, p. 207‑211, mars 1951, doi: 10.1088/0370-1301/64/3/304.


    ========== INPUT ==========

    [Temperature]
        The temperature of the material in [K]

    ========== OUTPUT ==========

    [ThermalConductivity]
        The thermal conductivity in [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ###############################################

    import numpy as np

    ################## CONDITIONS ############################################

    if Temperature <= 20 and Temperature >= 2:

        return 2 * 1e-2 * Temperature ** 1.4

        ################## SINON NAN #########################################

    else:

        print('Warning: The thermal conductivity of Ni60Cr15Fe16Mo7 is not defined for T = ' + str(Temperature) + ' K')
        return np.nan



#%%
def molar_volume(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the molar volume of a mole of Helium 4

    ========== VALIDITY ==========

    <temperature> : [0.05 -> 1.8]
    <pressure> : [0 -> 15.7e5]


    ========== FROM ==========

    Tanaka et al (2000)
        Molar volume of pure liquide 4He : dependence on temperature 
        (50-1000mK) and pressure (0-1.57 MPa) - Equation (25)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <molar_volume>
        -- float --
        The molar volume of Helium 4
        [m]**3.[mol]**(-1)

    ========== STATUS ==========

    Status : Checked 2022-12-24
    
    ========== UPDATE ==========
    
    2021-05-07:
        On Chaudhry et al. (2012) - P59, we can continue up to 1.8K with
        an error of 0.1% 

    """

    ################## MODULES ################################################

    import numpy as np
    from scipy.special import erfc

    ################## CONDITIONS #############################################

    assert 15.7e5 >= pressure >= 0, 'The function Helium4.molar_volume is not defined for P = ' + str(pressure) + ' Pa'
    assert 1.8 >= temperature >= 0.05, 'The function Helium4.molar_volume is not defined for T = ' + str(temperature) + 'K '

    ################## INITIALISATION #########################################

    V = np.array([2.757930e-5, -3.361585e-12, 1.602419e-18,-1.072604e-24,7.979064e-31, -5.356076e-37, 2.703689e-43, -9.004790e-50,1.725962e-56,-1.429411e-63])
    D = np.array([8.618, -7.487e-7,5.308e-14])
    A = np.array([1.863e8,7.664e6])
    B = np.array([2.191,-1.702e-7])
    pc = -1.00170e6
    
    result_polynome_A = 0
    result_polynome_B = 0
    result_polynome_D = 0
    result_polynome_V = 0
    

    ################## FUNCTION ###############################################

    for i in range(len(V)):
        result_polynome_V = result_polynome_V + V[i] * pressure ** i
    
    for i in range(len(D)):
        result_polynome_D = result_polynome_D + D[i] * pressure ** i
        
    result_polynome_A = (A[0] / ((pressure - pc) ** 2) + A[1] / ((pressure - pc) ** (5 / 3)))

    result_polynome_B =  B[0] + B[1] * pressure

    result = result_polynome_V*(1+result_polynome_A*temperature**4/4-result_polynome_B*erfc(np.sqrt((result_polynome_D/temperature))))
    
    return result

#%%
def mass():
    """
    ========== DESCRIPTION ==========

    Return the constant mass of a single atom of helium 4

    ========== VALIDITY ==========

    ========== FROM ==========

    ========== OUTPUT ==========

    <mass>
        -- float --
    	The mass of a single atom of Helium 4
        [kg]

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ################################################

    from cryopy import Constant
    from cryopy.Helium import Helium4

    ################## RETURN #################################################

    return Helium4.molar_mass() / Constant.avogadro()

#%%
def thermal_expansion_phonon(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the contribution of phonons to the thermal expansion of helium 4

    ========== VALIDITY ==========

    <temperature> : [0.05 -> 1.8]
    <pressure> : [0 -> 15.7e5]


    ========== FROM ==========

    Tanaka et al (2000) 
        Molar volume of pure liquide 4He : dependence on temperature 
        (50-1000mK) and pressure (0-1.57 MPa) - Equation (23)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <thermal_expansion_phonon>
        -- float --
        The thermal expansion of helium 4 due to phonons 
        [K]**{-1}

    ========== STATUS ==========

    Status : 
    
    ========== UPDATE ==========

    """

    ################## MODULES ################################################

    import numpy as np

    ################## CONDITIONS #############################################

    assert 15.7e5 >= pressure >= 0, 'The function Helium4.molar_volume is not defined for P = ' + str(pressure) + ' Pa'
    assert 1.8 >= temperature >= 0.05, 'The function Helium4.molar_volume is not defined for T = ' + str(temperature) + 'K '

    ################## INITIALISATION #########################################

    A = np.array([1.863e8,7.664e6])
    pc = -1.00170e6

    ################## FUNCTION ###############################################

    result = (A[0] / ((pressure - pc) ** 2) + A[1] / ((pressure - pc) ** (5 / 3)))

    return result

#%%
def thermal_expansion_roton(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the contribution of rotons to the thermal expansion of helium 4

    ========== VALIDITY ==========

    <temperature> : [0.05 -> 1.8]
    <pressure> : [0 -> 15.7e5]


    ========== FROM ==========

    TANAKA (2000) - Molar volume of pure liquide 4He : dependence on 
    temperature (50-1000mK) and pressure (0-1.57 MPa) - Equation (24)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <thermal_expansion_roton>
        -- float --
        The thermal expansion of helium 4 due to rotons 
        [K]**{-1}

    ========== STATUS ==========

    Status : 
    
    ========== UPDATE ==========

    """

    ################## MODULES ################################################

    import numpy as np

    ################## CONDITIONS #############################################

    assert 15.7e5 >= pressure >= 0, 'The function Helium4.molar_volume is not defined for P = ' + str(pressure) + ' Pa'
    assert 1.8 >= temperature >= 0.05, 'The function Helium4.molar_volume is not defined for T = ' + str(temperature) + 'K '

    ################## INITIALISATION #########################################

    D = np.array([8.618, -7.487e-7,5.308e-14])
    B = np.array([2.191,-1.702e-7])
    result_polynome_D = 0
    
    ################## FUNCTION ###############################################
    
    for i in range(len(D)):
        result_polynome_D = result_polynome_D + D[i] * pressure ** i

    result_polynome_B =  B[0] + B[1] * pressure
    
    result_b = result_polynome_B*np.sqrt(np.pi/result_polynome_D)

    result = -result_b*temperature**(-3/2)*np.exp(-result_polynome_D/temperature)
    
    return result


#%%
def molar_mass():
    """
    ========== DESCRIPTION ==========

    Return the constant molar mass of a single mole of helium 4

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    P. J. Mohr, B. N. Taylor, and D. B. Newell, Rev. Mod. Phys. 84, 1531 (2012).

    ========== OUTPUT ==========

    <molar_mass>
        -- float --
        The molar mass of a single mole of helium 4
        [kg].[mol]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## RETURN #################################################

    return 4.002602e-3


#%%
def density(temperature, pressure):
    """
    ========== DESCRIPTION ==========
    
    This function return the density of pure helium 4

    ========== VALIDITY ==========

    ========== FROM ==========

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]
        
    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]
        
    ========== OUTPUT ==========
    
    <density>
        -- float --
        The density of Helium 4
        [kg].[m]**3
    
    ========== STATUS ==========     
    
    Status : Checked

    """

    ################## CONDITIONS #############################################

    ################## INITIALISATION #########################################

    ################## FUNCTION ###############################################

    return molar_mass()/molar_volume(temperature,pressure)

#%%
def molar_specific_heat(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the molar specific heat of pure Fermi liquid helium-3

    ========== VALIDITY ==========

    <temperature> : [0 -> 1.8]
    <pressure> : [0]

    ========== FROM ==========

    KUERTEN - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Equation (46)
   
    CHAUDHRY - Thermodynamic properties of liquid 3He-4he mixtures
    between 0.15 K and 1.8 K - Equation (A15)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium-3
        [K]

    <pressure>
        -- float --
        The pressure of helium-3
        [Pa]

    ========== OUTPUT ==========

    <molar_specific_heat>
        -- float --
        The molar specific heat of pure Fermi liquid helium-3
        [J].[K]**(-1).[mol]**(-1)

    ========== STATUS ==========

    Status : Checked

    ========== NOTES ===========

    This function requires the implementation of pressure changes


    """

    ################## CONDITIONS #############################################

    assert pressure <= 0 <= pressure, 'The function Helium4.molar_specific_heat is not defined for P = ' + str(pressure) + ' Pa'
    assert 1.8 >= temperature >= 0.000, 'The function Helium4.molar_specific_heat is not defined for T = ' + str(temperature) + ' K'

    ################## MODULES ################################################

    import numpy as np
    import pandas as pd

    ################## INITIALISATION #########################################

    result = 0
    coefficients = pd.DataFrame(np.array([[0.08137],
                                          [0],
                                          [-0.0528],
                                          [0.05089],
                                          [0.019]]),
                                columns=['B'])

    ################## FUNCTION ###############################################

    if 0.4 >= temperature >= 0:
        for j in [0, 1, 2, 3, 4]:
            result = result + coefficients.B[j] * temperature ** (j + 3)
        return result

    else:
        first = 82.180127 * temperature ** 3 - 87.45899 * temperature ** 5 + 129.12758 * temperature ** 6 - 6.6314726 * temperature ** 7
        second = 70198.836 * (8.8955141 / temperature) ** (3 / 2) * np.exp(-8.8955141 / temperature) * (
                1 + (temperature / 8.8955141) + 0.75 * ((temperature / 8.8955141) ** 2))
        third = (10244.198 / temperature) * (22.890183 / temperature) ** 2 * np.exp(-22.890183 / temperature) * (
                1 - 2 * (temperature / 22.890183))
        value = (first + second + third) * 1e-3

        return value


#%%
def molar_enthalpy(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function can return the molar enthalpy of Helium 4

    ========== VALIDITY ==========

    <temperature> : [0 -> 0.400]
    <pressure> : [0]

    ========== FROM ==========
    
    KUERTEN - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Table (17)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <molar_enthalpy>
        -- float --
        The molar enthalpy of pure Helium 4
        [J].[mol]**(-1)


    ========== STATUS ==========

    Status : Checked

    ========== NOTES ===========

    This function requires the implementation of pressure changes


    """

    ################## CONDITIONS #############################################

    assert pressure <= 0 <= pressure, 'The function Helium4.molar_enthalpy is not defined for P = ' + str(pressure) + ' Pa'
    assert 0.400 >= temperature >= 0.000, 'The function  Helium4.molar_enthalpy is not defined for T = ' + str(temperature) + ' K'

    ################## MODULES ################################################

    import numpy as np

    ################## INITIALISATION #########################################

    coefficients = [3.19794159e-03, -6.48309258e-03, 2.19933824e-02, -2.09571258e-04,
                    1.23491357e-05, -2.67718222e-07, 1.07444578e-09]
    polynom = np.poly1d(coefficients)

    ################## FUNCTION ###############################################

    return polynom(temperature)

#%%
def molar_entropy(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function can return the molar entropy of Helium 4

    ========== VALIDITY ==========

    <temperature> : [0 -> 0.400]
    <pressure> : [0]

    ========== FROM ==========

    KUERTEN - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Table (17)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <molar_entropy>
        -- float --
        The molar entropy of pure Helium 4
        [J].[K]**(-1).[mol]**(-1)


    ========== STATUS ==========

    Status : Checked

    ========== NOTES ===========

    This function requires the implementation of pressure changes


    """

    ################## MODULES ################################################

    import numpy as np

    ################## CONDITIONS #############################################

    assert pressure <= 0 <= pressure, 'The function Helium3.molar_entropy is not defined for P = ' + str(pressure) + ' Pa'
    assert 0.400 >= temperature >= 0.000, 'The function Helium3.molar_entropy is not defined for T = ' + str(temperature) + ' K'

    ################## INITIALISATION #########################################

    coefficients = [-2.08468571e-03, 2.73347674e-03,
                    -5.46990881e-03, 2.81129291e-02,
                    -7.81349812e-05, 2.17304932e-06,
                    -1.05591438e-08]
    polynom = np.poly1d(coefficients)

    ################## FUNCTION ###############################################

    return polynom(temperature)

#%% 
def chemical_potential(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the chemical potential of Helium 4

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    KUERTEN - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Equation (38)
    
    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <chemical_potential>
        -- float --
        The chemical potential of pure Helium 4
        [J].[mol]**(-1)

    ========== STATUS ==========

    Status : In progress (need to be verified with new values of pressure)

    """

    ################## MODULES ################################################

    from cryopy.Helium import Helium4

    ################## RETURN #################################################

    return Helium4.molar_enthalpy(temperature, pressure) - temperature * Helium4.molar_entropy(temperature, pressure)


def dynamic_viscosity(temperature):
    """
    ========== DESCRIPTION ==========
    
    This function return the dynamic viscosity of pure helium 4

    ========== VALIDITY ==========
    
    <temperature> : [273.15->1800]

    ========== FROM ==========
    
    KPetersen, H. (1970). The properties of helium: Density, specific heats, 
    viscosity, and thermal conductivity at pressures from 1 to 100 bar and 
    from room temperature to about 1800 K. Risø National Laboratory. Denmark. 
    Forskningscenter Risoe. Risoe-R No. 224

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]
        
    ========== OUTPUT ==========
    
    <dynamic_viscosity>
        -- float --
        The dynamic viscosity of Helium 4
        [Pa].[s]
    
    ========== STATUS ==========     
    
    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert 1800 >= temperature >= 273.15, 'The function Helium4.dynamic_viscosity is not defined for T = ' + str(temperature) + ' K'

    ################## FUNCTION ###############################################

    return 3.674e-7 * temperature ** 0.7





# %%
def compressibility_factor(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the compressibility factor of pure helium 4

    ========== VALIDITY ==========

    <temperature> : [273.15->1800]
    <pressure> : [0->100e5]

    ========== FROM ==========

    KPetersen, H. (1970). The properties of helium: Density, specific heats,
    viscosity, and thermal conductivity at pressures from 1 to 100 bar and
    from room temperature to about 1800 K. Risø National Laboratory. Denmark.
    Forskningscenter Risoe. Risoe-R No. 224

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <compressibility_factor>
        -- float --
        The compressibility factor of Helium 4
        [Ø]

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert 1800 >= temperature >= 273.15, 'The function Helium4.compressibility_factor is not defined for T = ' + str(temperature) + ' K'
    assert 100e5 >= pressure >= 0, 'The function Helium4.compressibility_factor is not defined for P = ' + str(temperature) + ' Pa'

    ################## MODULES ################################################

    ################## INITIALISATION #########################################

    # convert [Pa] to [Bar]
    pressure = pressure * 1e-5

    ################## FUNCTION ###############################################

    return 1 + 0.4446 * pressure / (temperature ** 1.2)


# %%
def thermal_conductivity(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the compressibility factor of pure helium 4

    ========== VALIDITY ==========

    <temperature> : [273.15->1800]
    <pressure> : [0->100e5]

    ========== FROM ==========

    KPetersen, H. (1970). The properties of helium: Density, specific heats,
    viscosity, and thermal conductivity at pressures from 1 to 100 bar and
    from room temperature to about 1800 K. Risø National Laboratory. Denmark.
    Forskningscenter Risoe. Risoe-R No. 224

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <thermal_conductivity>
        -- float --
        The thermal conductivity of Helium 4
        [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert 1800 >= temperature >= 273.15, 'The function Helium4.thermal_conductivity is not defined for T = ' + str(temperature) + ' K'
    assert 100e5 >= pressure >= 0, 'The function Helium4.thermal_conductivity is not defined for P = ' + str(temperature) + ' Pa'

    ################## INITIALISATION #########################################

    # convert [Pa] to [Bar]
    pressure = pressure * 1e-5

    ################## FUNCTION ###############################################

    return 2.682e-3 * (1 + 1.123e-3 * pressure) * temperature ** (0.71 * (1 - 2e-4 * pressure))


# %%
def prandtl_number(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the Prandtl number of pure Helium 4

    ========== VALIDITY ==========

    <temperature> : [273.15->1800]
    <pressure> : [0->100e5]

    ========== FROM ==========

    KPetersen, H. (1970). The properties of helium: Density, specific heats,
    viscosity, and thermal conductivity at pressures from 1 to 100 bar and
    from room temperature to about 1800 K. Risø National Laboratory. Denmark.
    Forskningscenter Risoe. Risoe-R No. 224

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <prandtl_number>
        -- float --
        The Prandtl number of pure Helium 4
        [Ø]

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS ############################################

    assert 1800 >= temperature >= 273.15, 'The function Helium4.prandtl_number is not defined for T = ' + str(temperature) + ' K'
    assert 100e5 >= pressure >= 0, 'The function Helium4.prandtl_number is not defined for P = ' + str(temperature) + ' Pa'

    ################## INITIALISATION ########################################

    # convert [Pa] to [Bar]
    pressure = pressure * 1e-5

    ################## FUNCTION ##############################################

    return 0.7117 / (1 + 1.123e-3 * pressure) * temperature ** (-0.01 * 1.42e-4 * pressure)


# %%
def is_liquid(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function determine if the current Helium 4 is liquid or not

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <is_liquid>
        -- boolean --
        Helium 4 is liquid or not
        []

    ========== STATUS ==========

    Status : Checked

    """

    ################## FUNCTION ##############################################

def is_solid(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function determine if the current Helium 4 is solid or not

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <is_solid>
        -- boolean --
        Helium 4 is solid or not
        []

    ========== STATUS ==========

    Status : Checked

    """

    ################## FUNCTION ##############################################


def is_gaseous(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function determine if the current Helium 4 is gaseous or not

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <is_gaseous>
        -- boolean --
        Helium 4 is gaseous or not
        []

    ========== STATUS ==========

    Status : Checked

    """

    ################## FUNCTION ##############################################


def is_superfluid(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function determine if the current Helium 4 is superfluid or not

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <is_superfluid>
        -- boolean --
        Helium 4 is superfluid or not
        []

    ========== STATUS ==========

    Status : Checked

    """


def superfluid_ratio(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function determine the current proportion of superfluid Helium 4

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]

    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]

    ========== OUTPUT ==========

    <superfluid_ratio>
        -- float --
        Proportion of superfluid Helium 4
        []

    ========== STATUS ==========

    Status : Checked

    """




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# %%
def thermal_expansion(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal expansion coefficient of STYCAST 2850 FT cat 9

    ========== VALIDITY ==========

    <temperature> : [4.2 -> 335]

    ========== FROM ==========

    Internal report - contact valentin.sauvage@outlook.com 

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <thermal_expansion>
        -- float --
        The thermal expansion coefficient of stycast 2850FT cat 9
        [Ø]

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert temperature <= 335 and temperature >= 4.2, 'The function ' \
                                                      ' STYCAST2850FT9.thermal_expansion is not defined for ' \
                                                      'T = ' + str(temperature) + ' K'

    ################## FUNCTION ###############################################

    expansion = 4.694544e-7 * temperature ** 4 - 3.352487e-4 * temperature ** 3 + 1.211048e-1 * temperature ** 2 - 2.852325 * temperature - 4.609941e3

    return expansion * 1e-6



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Dec  2 14:05:48 2021

@author: valentinsauvage
"""


def All():
    """
    ========== DESCRIPTION ==========

    This function plot the thermal conductivity of cryopy referenced materials 

    ========== STATUS ==========

    Status : Checked

    """

    import matplotlib.pyplot as plt
    # from cryopy.material import *
    import numpy as np

    plt.figure()

    Temperature = np.arange(3, 300, 0.1)
    ThermalConductivity = [Teflon.ThermalConductivity(T) for T in Temperature]

    plt.loglog(Temperature, ThermalConductivity)

    plt.xlabel('Temperature [K]')
    plt.ylabel(r'Thermal Conductivity $[W.m^{-1}.K^{-1}]$')
    plt.grid()
    plt.title(r'Thermal Conductivity of some materials in \textit{cryopy}')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# %%
def ThermalConductivity(Temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of Vespel SP1
    
    ========== Validity ==========

    1.4 K < Temperature < 45 K

    ========== FROM ==========

    L Duband, A Lange, et J. J. Bock. « Helium adsorption coolers for space ». 
    Submillimetre and Far-Infrared Space Instrumentation, Proceedings of the 
    30th ESLAB Symposium held in Noordwijk, 24-26 September 1996. 
    Edited by E.J. Rolfe and G. Pilbratt. ESA SP-388. 
    Paris: European Space Agency, 1996., p.289, s. d.

    ========== INPUT ==========

    [Temperature]
        The temperature of the material in [K]

    ========== OUTPUT ==========

    [ThermalConductivity]
        The thermal conductivity in [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ###############################################

    import numpy as np

    ################## CONDITIONS ############################################

    if Temperature <= 45 and Temperature >= 1.4:

        ################## INITIALISATION ####################################

        ################## IF CONDITION TRUE #####################

        return 1.7e-3 * Temperature ** 1.18

        ################## SINON NAN #########################################

    else:

        print('Warning: The thermal conductivity of Vespel SP1 is not defined for T = ' + str(Temperature) + ' K')
        return np.nan



#!/usr/bin/env python3

# %%
def query_identification(address):
    """
    ========== DESCRIPTION ==========

    This function can return the identification of the Cryocon Model 54

    ========== FROM ==========

    Manual of Cryo-con Model 54

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')

    ========== OUTPUT ==========

    <manufacturer>
        -- string --
        Should be "Cryo-con"

    <model>
        -- string --
        Should be "Model 54"

    <serial>
        -- string --
        Depend on your instrument

    <firmware_version>
        -- string --
        Depend on your instrument

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========

    from cryopy import Instrument.Cryocon.Model54

    manufacturer,model,serial,firmware_version = Model54.query_identification('GPIB0::15::INSTR')

    """

    ################## MODULES ###############################################

    import pyvisa

    ################## INITIALISATION ########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    answer = instru.query('*IDN?')

    manufacturer = answer[0:8]
    model = answer[9:17]
    serial = answer[18:24]
    firmware_version = answer[25:30]

    return manufacturer, model, serial, firmware_version


# %%
def query_temperature(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can return the temperature of a given channel of the Cryocon Model 54

    ========== FROM ==========

    Manual of Cryo-con Model 54

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')

    <channel>
        -- string --
        The channel ('a','b','c' or 'd')

    ========== OUTPUT ==========

    <temperature>
        -- float --
        The temperature measured by the channel
        [K]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========

    from cryopy import Instrument.Cryocon.Model54

    temperature= Model54.query_temperature('GPIB0::15::INSTR','a')

    """

    ################## MODULES ###############################################

    import pyvisa
    import numpy

    ################## INITIALISATION ########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    answer = instru.query('input? ' + channel)

    try:
        temperature = float(answer)
        return temperature

    except ValueError:
        return numpy.nan



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# %%
def thermal_conductivity(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of Cu70Ni30

    ========== VALIDITY ==========

    <temperature> : [0.05 -> 4]

    ========== FROM ==========


    C. Y. Ho, M. W. Ackerman, K. Y. Wu, S. G. Oh, et T. N. Havill,
    « Thermal conductivity of ten selected binary alloy systems »,
    Journal of Physical and Chemical Reference Data, vol. 7, no 3,
    p. 959‑1178, juill. 1978, doi: 10.1063/1.555583.

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <thermal_conductivity>
        -- float --
        The thermal conductivity
        [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert temperature <= 4 and temperature >= 0.05, 'The function ' \
                                                     ' Cu70Ni30.thermal_conductivity is not defined for ' \
                                                     'T = ' + str(temperature) + ' K'

    ################## FUNCTION ###############################################

    if temperature >= 0.3:
        result = 0.093 * temperature ** 1.23
    else:
        0.064 * temperature

    return result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# %%
def query_measurement(address):
    """
    ========== DESCRIPTION ==========

    This function can query the measurement value

    ========== FROM ==========

    Manual of Keithley 192

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
     
    ========== OUTPUT ==========
    
    <measurement>
        -- float --
        The measurement
        [depend]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    answer = instru.read()

    measurement = float(answer[4:16])

    ################## FUNCTION ###############################################

    return voltage


# %%
def command_function(address, function):
    """
    ========== DESCRIPTION ==========

    This function can command the function

    ========== FROM ==========

    Manual of Keithley 192

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <function>
        -- int --
        The function (0     =   DV Volts,
                      1     =   AC Volts,
                      2     =   Ohms,
                      3     =   DC current,
                      4     =   AC current,
                      5     =   ACV dB,
                      6     =   ACA dB,
                      7     =   Offset compensation Ohms)

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    function = str(function)

    instruction = 'F' + function + 'X'

    instru.write(instruction)

    ################## FUNCTION ###############################################

    return


# %%
def command_range(address, range_value):
    """
    ========== DESCRIPTION ==========

    This function can command the range_value

    ========== FROM ==========

    Manual of Keithley 192

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <range_value>
        -- int --
        The function (0     =   Autorange,
                      1     =   specific value,
                      2     =   specific value,
                      3     =   specific value,
                      4     =   specific value,
                      5     =   specific value,
                      6     =   specific value,
                      7     =   specific value)

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    range_value = str(range_value)

    instruction = 'F' + range_value + 'X'

    instru.write(instruction)

    ################## FUNCTION ###############################################

    return



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# %%
def thermal_conductivity(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of Cu-OFHC

    ========== VALIDITY ==========

    <temperature> : [4 -> 300]

    ========== FROM ==========

    E. D. Marquardt, J. P. Le, et R. Radebaugh, « Cryogenic Material
    Properties Database », p. 7, 2000.

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <thermal_conductivity>
        -- float --
        The thermal conductivity
        [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert temperature <= 400 and temperature >= 4, 'The function ' \
                                                    ' Cu-OFHC.thermal_conductivity is not defined for ' \
                                                    'T = ' + str(temperature) + ' K'

    ################## FUNCTION ###############################################

    result = (
                         2.2154 - 0.88068 * temperature ** 0.5 + 0.29505 * temperature - 0.048310 * temperature ** 1.5 + 0.003207 * temperature ** 2) / (
                         1 - 0.47461 * temperature ** 0.5 + 0.13871 * temperature - 0.020430 * temperature ** 1.5 + 0.001281 * temperature ** 2)

    return 10 ** result


# %%
def mass_specific_heat(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the specific heat of Cu-OFHC

    ========== VALIDITY ==========

    <temperature> : [3 -> 300]

    ========== FROM ==========

    E. D. Marquardt, J. P. Le, et R. Radebaugh, « Cryogenic Material
    Properties Database », p. 7, 2000.

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <specific_heat>
        -- float --
        The specific heat
        [J].[Kg]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert temperature <= 300 and temperature >= 3, 'The function ' \
                                                    ' Cu-OFHC.mass_specific_heat is not defined for ' \
                                                    'T = ' + str(temperature) + ' K'

    ################## MODULES ################################################

    import numpy as np

    ################## INITIALISATION #########################################

    coefficients = np.array([-1.91844, -0.15973, 8.61013, -18.99640, 21.96610, -12.73280, 3.54322, -0.37970, 0])

    ################## FUNCTION ###############################################

    result = 0

    for i in range(len(coefficients)):
        result = result + coefficients[i] * np.log10(temperature) ** i

    return 10 ** result



def planck():
    """
    ========== DESCRIPTION ==========

    This function return the Planck constant

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    Wikipedia : https://en.wikipedia.org/wiki/Planck_constant

    ========== OUTPUT ==========

    <planck>
        -- float --
        The Planck constant
        [kg].[m]**(2).[s]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    # RETURN

    return 6.626070040e-34


def reduced_planck():
    """
    ========== DESCRIPTION ==========

    This function return the reduced Planck constant

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    Wikipedia : https://en.wikipedia.org/wiki/Planck_constant

    ========== OUTPUT ==========

    <reduced_planck>
        -- float --
        The reduced Planck constant
        [kg].[m]**(2).[s]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    # MODULES

    import numpy as np
    import constant

    # RETURN

    return constant.planck() / 2 / np.pi


def gas():
    """
    ========== DESCRIPTION ==========

    This function return the molar gas constant

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    Wikipedia : https://en.wikipedia.org/wiki/Gas_constant

    ========== INPUT ==========

    ========== OUTPUT ==========

    <gas>
        -- float --
        [J].[K]**(-1).[mol]**(-1)
        The molar gas constant

    ========== STATUS ==========

    Status : Checked

    """

    # RETURN

    return 8.31446261815324


def boltzmann():
    """
    ========== DESCRIPTION ==========

    This function return the Boltzmann constant

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    Wikipedia : https://en.wikipedia.org/wiki/Boltzmann_constant

    ========== OUTPUT ==========

    <boltzmann>
        -- float --
        The Boltzmann constant
        [J].[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    # RETURN

    return 1.38064852e-23


def avogadro():
    """
    ========== DESCRIPTION ==========

    This function return the Avogadro constant

    ========== VALIDITY ==========

    Always

    ========== SOURCE ==========

    Wikipedia : https://en.wikipedia.org/wiki/Avogadro_constant

    ========== INPUT ==========

    ========== OUTPUT ==========

    <avogadro>
        -- float --
        The Avogadro constant
        [mol]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    # RETURN

    return 6.0221367e23


def speed_of_light():
    """
    ========== DESCRIPTION ==========

    This function return the speed of light in vacuum

    ========== VALIDITY ==========

    Only in Vacuum

    ========== FROM ==========

    Wikipedia : https://en.wikipedia.org/wiki/Speed_of_light

    ========== INPUT ==========

    ========== OUTPUT ==========

    <speed_of_light>
        -- int --
        The speed of light
        [m].[s]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    # RETURN

    return 299792458


def stefan_boltzmann():
    """
    ========== DESCRIPTION ==========

    This function return the Stefan-Boltzmann constant

    ========== VALIDITY ==========

    Always

    ========== FROM ==========

    Wikipedia : https://en.wikipedia.org/wiki/Stefan–Boltzmann_law

    ========== INPUT ==========

    ========== OUTPUT ==========

    <stefan_boltzmann>
        -- float --
        The Stefan-Boltzmann constant
        [W].[m]**(-2).[K]**(-4)

    ========== STATUS ==========

    Status : Checked

    """

    # MODULES

    import numpy as np
    from . import constant

    # RETURN

    return 2 * np.pi ** 5 * constant.boltzmann() ** 4 / (15 * constant.speed_of_light() ** 2 * constant.planck() ** 3)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# %%
def thermal_conductivity(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of Ti90Al6V4 (TA6V)

    ========== VALIDITY ==========

    <temperature> : [20 -> 300]

    ========== FROM ==========

    E. D. Marquardt, J. P. Le, et R. Radebaugh, « Cryogenic Material
    Properties Database », p. 7, 2000.

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <thermal_conductivity>
        -- float --
        The thermal conductivity
        [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## PACKAGES ###############################################

    import numpy as np

    ################## CONDITIONS #############################################

    assert temperature <= 300 and temperature >= 20, 'The function ' \
                                                     ' Ti90Al6V4.thermal_conductivity is not defined for ' \
                                                     'T = ' + str(temperature) + ' K'

    ################## INITIALISATION #########################################

    coefficients = np.array(
        [-5107.8774, 19240.422, -30789.0064, 27134.756, -14226.379, 4438.2154, -763.07767, 55.796592])
    result = 0

    ################## FUNCTION ###############################################

    for i in range(len(coefficients)):
        result = result + coefficients[i] * np.log10(temperature) ** i

    return 10 ** result


# %%
def linear_thermal_expansion(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the linear thermal expansion of Ti90Al6V4 (TA6V)

    ========== VALIDITY ==========

    <temperature> : [4 -> 300]

    ========== FROM ==========

    E. D. Marquardt, J. P. Le, et R. Radebaugh, « Cryogenic Material
    Properties Database », p. 7, 2000.

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <linear_thermal_expansion>
        -- float --
        The linear thermal expansion
        []

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert temperature <= 300 and temperature >= 4, 'The function ' \
                                                    ' Ti90Al6V4.linear_thermal_expansion is not defined for ' \
                                                    'T = ' + str(temperature) + ' K'

    ################## MODULES ################################################

    import numpy as np

    ################## INITIALISATION #########################################

    coefficients = np.array([-1.711e2, -2.171e-1, 4.841e-3, -7.202e-6, 0])
    result = 0

    ################## FUNCTION ###############################################

    for i in range(len(coefficients)):
        result = result + coefficients[i] * temperature ** i

    return result * 1e-5



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# %%
def thermal_conductivity(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of Cu84Ni4Mn12 (Manganin)

    ========== VALIDITY ==========

    <temperature> : [0.1 -> 4]

    ========== FROM ==========

    I. Peroni, E. Gottardi, A. Peruzzi, G. Ponti, et G. Ventura,
    « Thermal conductivity of manganin below 1 K », Nuclear Physics B -
    Proceedings Supplements, vol. 78, no 1‑3, p. 573‑575, août 1999,
    doi: 10.1016/S0920-5632(99)00606-4.

    D.T. Corzett, A.M.Miller and P.Seligmann, Cryogenics 16,505 (1976)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <thermal_conductivity>
        -- float --
        The thermal conductivity
        [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert temperature <= 4 and temperature >= 0.1, 'The function ' \
                                                    ' Cu84Ni4Mn12.thermal_conductivity is not defined for ' \
                                                    'T = ' + str(temperature) + ' K'

    ################## FUNCTION ###############################################

    result = 0.095 * temperature ** 1.19

    return result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Dec 24 00:31:14 2022

@author: valentinsauvage
"""
#%% 
def density(temperature, pressure):
    """
    ========== DESCRIPTION ==========
    
    This function return the density of pure helium 4

    ========== VALIDITY ==========
    
    <temperature> : [273.15->1800]
    <pressure> : [0->100e5]

    ========== FROM ==========
    
    KPetersen, H. (1970). The properties of helium: Density, specific heats, 
    viscosity, and thermal conductivity at pressures from 1 to 100 bar and 
    from room temperature to about 1800 K. Risø National Laboratory. Denmark. 
    Forskningscenter Risoe. Risoe-R No. 224

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]
        
    <pressure>
        -- float --
        The pressure of Helium 4
        [Pa]
        
    ========== OUTPUT ==========
    
    <density>
        -- float --
        The density of Helium 4
        [kg].[m]**3
    
    ========== STATUS ==========     
    
    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert 1800 >= temperature >= 273.15, 'The function Helium4.density is not defined for T = ' + str(temperature) + ' K'
    assert 100e5 >= pressure >= 0, 'The function Helium4.density is not defined for P = ' + str(temperature) + ' Pa'

    ################## INITIALISATION #########################################

    # convert [Pa] to [Bar]
    pressure = pressure * 1e-5

    ################## FUNCTION ###############################################

    return 48.18 * pressure / temperature * (1 + 0.4446 * pressure / (temperature ** 1.2)) ** (-1)

#%%
def dynamic_viscosity(temperature):
    """
    ========== DESCRIPTION ==========
    
    This function return the dynamic viscosity of pure helium 4

    ========== VALIDITY ==========
    
    <temperature> : [273.15->1800]

    ========== FROM ==========
    
    KPetersen, H. (1970). The properties of helium: Density, specific heats, 
    viscosity, and thermal conductivity at pressures from 1 to 100 bar and 
    from room temperature to about 1800 K. Risø National Laboratory. Denmark. 
    Forskningscenter Risoe. Risoe-R No. 224

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of Helium 4
        [K]
        
    ========== OUTPUT ==========
    
    <dynamic_viscosity>
        -- float --
        The dynamic viscosity of Helium 4
        [Pa].[s]
    
    ========== STATUS ==========     
    
    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert 1800 >= temperature >= 273.15, 'The function Helium4.dynamic_viscosity is not defined for T = ' + str(temperature) + ' K'

    ################## FUNCTION ###############################################

    return 3.674e-7 * temperature ** 0.7




#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# %%
def ThermalConductivity(Temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of Teflon
    
    ========== Validity ==========

    4K < Temperature < 300K

    ========== FROM ==========
    
    E. D. Marquardt, J. P. Le, et R. Radebaugh, « Cryogenic Material Properties Database », p. 7, 2000.

    ========== INPUT ==========

    [Temperature]
        The temperature of the material in [K]

    ========== OUTPUT ==========

    [ThermalConductivity]
        The thermal conductivity in [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ###############################################

    import numpy as np

    ################## CONDITIONS ############################################

    if Temperature <= 300 and Temperature >= 4:

        ################## INITIALISATION ####################################

        Coefficients = [2.7380, -30.677, 89.430, -136.99, 124.69, -69.556, 23.320, -4.3135, 0.33829]
        Sum = 0
        ################## IF CONDITION TRUE #####################

        for i in range(len(Coefficients)):
            Sum = Sum + Coefficients[i] * np.log10(Temperature) ** i

        return 10 ** Sum

        ################## SINON NAN #########################################

    else:

        print('Warning: The thermal conductivity of Teflon is not defined for T = ' + str(Temperature) + ' K')
        return np.nan


# %%
def SpecificHeat(Temperature):
    """
    ========== DESCRIPTION ==========

    This function return the specific heat of Teflon
    
    ========== Validity ==========

    3K < Temperature < 300K

    ========== FROM ==========
    
    E. D. Marquardt, J. P. Le, et R. Radebaugh, « Cryogenic Material Properties Database », p. 7, 2000.

    ========== INPUT ==========

    [Temperature]
        The temperature of the material in [K]

    ========== OUTPUT ==========

    [ThermalConductivity]
        The thermal conductivity in [J].[kg]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ###############################################

    import numpy as np

    ################## CONDITIONS ############################################

    if Temperature <= 300 and Temperature >= 3:

        ################## INITIALISATION ####################################

        Coefficients = [31.8825, -166.519, 352.019, 259.981, -104.614, 24.9927, -3.20792, 0.165032, 0]
        Sum = 0
        ################## IF CONDITION TRUE #####################

        for i in range(len(Coefficients)):
            Sum = Sum + Coefficients[i] * np.log10(Temperature) ** i

        return Sum

        ################## SINON NAN #########################################

    else:

        print('Warning: The specitif heat of Teflon is not defined for T = ' + str(Temperature) + ' K')
        return np.nan


# %%
def LinearThermalExpansion(Temperature):
    """
    ========== DESCRIPTION ==========

    This function return the linear thermal expansion of Teflon
    
    ========== Validity ==========

    4K < Temperature < 300K

    ========== FROM ==========
    
    E. D. Marquardt, J. P. Le, et R. Radebaugh, « Cryogenic Material Properties Database », p. 7, 2000.

    ========== INPUT ==========

    [Temperature]
        The temperature of the material in [K]

    ========== OUTPUT ==========

    [LinearThermalExpansion]
        The linear thermal expansion in [%]

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ###############################################

    ################## CONDITIONS ############################################

    if Temperature <= 300 and Temperature >= 4:

        ################## INITIALISATION ####################################

        Coefficients = [-2.165e3, 3.278, -8.218e-3, 7.244e-5, 0]
        Sum = 0
        ################## IF CONDITION TRUE #####################

        for i in range(len(Coefficients)):
            Sum = Sum + Coefficients[i] * Temperature ** i

        return Sum * 1e-5

        ################## SINON NAN #########################################

    else:

        print('Warning: The linear thermal expansion of Teflon is not defined for T = ' + str(Temperature) + ' K')
        return np.nan



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# %%
def ThermalConductivity(Temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of Copper Beryllium
    
    ========== Validity ==========

    4K < Temperature < 300K

    ========== FROM ==========
    
    E. D. Marquardt, J. P. Le, et R. Radebaugh, « Cryogenic Material Properties Database », p. 7, 2000.

    ========== INPUT ==========

    [Temperature]
        The temperature of the material in [K]

    ========== OUTPUT ==========

    [ThermalConductivity]
        The thermal conductivity in [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ###############################################

    import numpy as np

    ################## CONDITIONS ############################################

    if Temperature <= 300 and Temperature >= 4:

        ################## INITIALISATION ####################################

        Coefficients = [-0.50015, 1.93190, -1.69540, 0.71218, 1.27880, -1.61450, 0.68722, -0.10501, 0]
        Sum = 0
        ################## IF CONDITION TRUE #####################

        for i in range(len(Coefficients)):
            Sum = Sum + Coefficients[i] * np.log10(Temperature) ** i

        return 10 ** Sum

        ################## SINON NAN #########################################

    else:

        print('Warning: The thermal conductivity of CuBe is not defined for T = ' + str(Temperature) + ' K')
        return np.nan


# %%
def LinearThermalExpansion(Temperature):
    """
    ========== DESCRIPTION ==========

    This function return the linear thermal expansion of Copper Beryllium
    
    ========== Validity ==========

    4K < Temperature < 300K

    ========== FROM ==========
    
    https://trc.nist.gov/cryogenics/materials/Beryllium%20Copper/BerylliumCopper_rev.htm

    ========== INPUT ==========

    [Temperature]
        The temperature of the material in [K]

    ========== OUTPUT ==========

    [LinearThermalExpansion]
        The linear thermal expansion in [%]

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ###############################################

    ################## CONDITIONS ############################################

    if Temperature <= 300 and Temperature >= 4:

        ################## INITIALISATION ####################################

        Coefficients = [-3.1150e2, -4.4498e-1, 1.0133e-2, -2.4718e-5, 2.6277e-8]
        Sum = 0
        ################## IF CONDITION TRUE #####################

        for i in range(len(Coefficients)):
            Sum = Sum + Coefficients[i] * Temperature ** i

        return Sum * 1e-5

        ################## SINON NAN #########################################

    else:

        print('Warning: The linear thermal expansion of CuBe is not defined for T = ' + str(Temperature) + ' K')
        return np.nan



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# %%
def thermal_conductivity(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of Cu97Be3

    ========== VALIDITY ==========

    <temperature> : [4.4 -> 300]

    ========== FROM ==========

    Internal report - contact valentin.sauvage@outlook.com

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <thermal_conductivity>
        -- float --
        The thermal conductivity
        [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## PACKAGES ###############################################

    import numpy as np

    ################## CONDITIONS #############################################

    assert temperature <= 300 and temperature >= 4.4, 'The function ' \
                                                      ' Cu97Be3.thermal_conductivity is not defined for ' \
                                                      'T = ' + str(temperature) + ' K'

    ################## INITIALISATION #########################################

    coefficients = np.array([-0.8517934, 2.22797618, -0.29595893, 0.01743955])
    result = 0

    ################## FUNCTION ###############################################

    for i in range(len(coefficients)):
        result = result + coefficients[i] * np.log(temperature) ** i

    return np.exp(result)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# %%
def permeability(volumetric_flowrate, temperature, dynamic_viscosity, length, diameter, inlet_pressure,
                 outlet_pressure):
    """
    ========== DESCRIPTION ==========

    This function return the permeability of a circular porous media.

    ========== VALIDITY ==========

    laminar flow -> reynolds_number < 2000 
    gas is compressible 
    
    ========== FROM ==========

    Whitaker, Stephen. « Flow in Porous Media I: A Theoretical Derivation of 
    Darcy’s Law ». Transport in Porous Media 1, nᵒ 1 (1986): 3‑25. 
    https://doi.org/10.1007/BF01036523 

    ========== INPUT ==========

    <volumetric_flowrate>
        -- float --
        The inlet flowrate of gas through the porous media
        [m]**3.[s]**(-1)

    <temperature>
        -- float --
        The temperature of the gas 
        [K]

    <dynamic_viscosity>
        -- float --
        The dynamic viscosity of the gas 
        [Pa].[s]
        
    <length>
        -- float --
        The length of the porous media
        [m]
        
    <diameter>
        -- float --
        The diameter of the porous media
        [m]
        
    <inlet_pressure>
        -- float --
        The upstream pressure of the gas 
        [Pa]
        
    <outlet_pressure>
        -- float --
        The downstream pressure of the gas
        [K]
        
    ========== OUTPUT ==========

    <permeability>
        -- float --
        The permeability of the porous media
        [m]**2

    ========== STATUS ==========

    Status : ... 

    ========== NOTES ===========

    """

    ################## CONDITIONS #############################################

    from cryopy import Flow
    from cryopy import Constant
    import numpy as np

    cross_section = np.pi * diameter ** 2 / 4

    flow_speed = molar_flow * Constant.GAS() * temperature / inlet_pressure / cross_section
    reynolds = Flow.reynolds_number(density, diameter, dynamic_viscosity, flow_speed)

    assert reynolds >= 2000, 'The function ' \
                             ' Flow.permeability is not defined for ' \
                             'Re = ' + str(reynolds)

    ################## FUNCTION ###############################################

    return -volumetric_flowrate * dynamic_viscosity * length / cross_section / (inlet_pressure - outlet_pressure)


# %%
def reynolds_number(density, characteristic_dimension, dynamic_viscosity, flow_speed):
    """
    ========== DESCRIPTION ==========

    This function return the Reynolds number of a flow

    ========== VALIDITY ==========

    <density> -> [all]
    <characteristic_dimension> -> [all]
    <dynamic_viscosity> -> [all]
    <flow_speed> -> [all]
    
    ========== FROM ==========

    Dude, trust me

    ========== INPUT ==========

    <density>
        -- float --
        The density of the flowing gas
        [kg].[m]**3

    <characteristic_dimension>
        -- float --
        The characteristic linear dimension of the flow 
        [m]

    <dynamic_viscosity>
        -- float --
        The dynamic viscosity of the gas 
        [Pa].[s]
        
    <flow_speed>
        -- float --
        The speed of the flow
        [m].[s]**(-1)
        
    ========== OUTPUT ==========

    <reynolds_number>
        -- float --
        The Reynolds number of the flow
        [Ø]

    ========== STATUS ==========

    Status : Checked

    ========== NOTES ===========

    """

    ################## FUNCTION ############################################### 

    return density * flow_speed * characteristic_dimension / dynamic_viscosity



def molar_specific_heat(temperature, pressure, fraction_3he):
    """
    ========== DESCRIPTION ==========

    This function return the molar specific heat of a mixture of helium 3 & helium 4
    
    ========== FROM ==========

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the mixture
        [K]

    <pressure>
        -- float --
        The pressure of the mixture
        [Pa]

    <fraction_3he>
        -- float --
        The fraction of helium 3 on the mixture
        []

    ========== OUTPUT ==========

    <molar_specific_heat>
        -- float --
        The molar specific heat of helium 3/4 mixture
        [J].[K]**(-1).[mol]**(-1)


    ========== STATUS ==========

    Status : Checked

    ========== NOTES ===========

    """

    ################## MODULES ################################################

    from cryopy.helium import helium4
    from cryopy.helium import fermi

    ################## FUNCTION ###############################################

    return fraction_3he * fermi.molar_specific_heat(temperature, pressure, fraction_3he) + (1 - fraction_3he) * helium4.molar_specific_heat(temperature, pressure)

#%%
def molar_volume(temperature, pressure, fraction_3he):
    """
    ========== DESCRIPTION ==========

    This function return the molar volume of a mixture of helium 3 & helium 4 

    ========== FROM ==========

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium 4
        [K]

    <pressure>
        -- float --
        The pressure of helium 4
        [Pa]

    <fraction_3he>
        -- float --
        The fraction of helium 3 on the mixture
        []

    ========== OUTPUT ==========

    <molar_volume>
        -- float --
        The molar volume of the mixture
        [m]**(3).[mol]**(-1)

    ========== STATUS ==========

    Status : Checked
    
    ========== NOTES ==========

    """

    ################## MODULES ################################################

    from cryopy.helium import helium4
    from cryopy.helium import helium3

    ################## FUNCTION ###############################################

    return fraction_3he*helium3.molar_volume(temperature, pressure) + (1-fraction_3he)*helium4.molar_volume(temperature, pressure)

#%%
def molar_mass(temperature, pressure,fraction_3he):
    """
    ========== DESCRIPTION ==========

    This function return the molar mass of a mixture of helium 3 & 
    helium 4 

    ========== FROM ==========

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the mixture
        [K]

    <pressure>
        -- float --
        The pressure of the mixture
        [Pa]

    <fraction_3he>
        -- float --
        The fraction of helium 3 on mixture
        []

    ========== OUTPUT ==========

    <molar_mass>
        -- float --
        The molar mass of the mixture
        [kg].[mol]**(-1)

    ========== STATUS ==========

    Status : 
    
    ========== NOTES ==========


    """

    ################## MODULES ################################################

    from cryopy.helium import helium4
    from cryopy.helium import helium3

    ################## FUNCTION ###############################################

    return fraction_3he*helium3.molar_mass(temperature,pressure)+(1-fraction_3he)*helium4.molar_mass(temperature,pressure)

#%%
def osmotic_pressure(temperature, pressure, fraction_3he):
    """
    ========== DESCRIPTION ==========

    This function return the osmotic pressure of a mixture of helium 3 & 
    helium 4 

    ========== FROM ==========

    Kuerten et al. (1987)
        Thermodynamic properties of liquid 3He-4He mixtures at zero pressure for temperatures below 250 mK and 3He concentrations below 8% - Equation (24)

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of helium 4
        [K]

    <pressure>
        -- float --
        The pressure of helium 4
        [Pa]

    <fraction_3he>
        -- float --
        The fraction of helium 3 on mixture
        []

    ========== OUTPUT ==========

    <osmotic_pressure>
        -- float --
        The osmotic pressure of the mixture
        [Pa]

    ========== STATUS ==========

    Status : Checked

    ========== NOTES ==========
    
    2020-02-25:
        Modification of value to T=0 by a polynôm using values from table (19)
        instead of equation (45)
    
    """

    ################## MODULES ################################################

    import numpy as np
    from cryopy.helium import helium4
    from cryopy.helium import fermi
    from cryopy.helium import helium7
    from scipy.misc import derivative
    from scipy.integrate import quad

    ################## INITIALISATION #########################################

    coefficients = [7.71714007e+09, -2.28854069e+09, 2.80349751e+08, -2.01610697e+07, 7238437e+06, 3.68490992e+03,
                    -4.60897007e-01]
    polynom = np.poly1d(coefficients)

    t = temperature / fermi.temperature_function(temperature, pressure, fraction_3he)

    # Temporary function for derivation of fermi temperature 
    def fun(fraction_3he):
        return fermi.temperature_function(temperature, pressure, fraction_3he)

    ################## FUNCTION ###############################################

    if temperature == 0:
        return polynom(fraction_3he)
    else:
        return helium7.osmotic_pressure(0, pressure, fraction_3he) + fraction_3he ** 2 / helium4.molar_volume(
            temperature, pressure) * derivative(fun, fraction_3he) * quad(fermi.molar_specific_heat, 0, t, fraction_3he)


#%%
def molar_entropy(temperature, pressure, fraction_3he):
    """
    ========== DESCRIPTION ==========


    This function return the molar entropy of a mixture of helium 3 & 
    helium 4 

    ========== VALIDITE ==========

    0 <= Temperature <= 2 K
    0 <= Pression <= 0 Pa
    0 <= Concentration3He <= 100 %

    ========== SOURCE ==========

    KUERTEN - Thermodynamic properties of liquid 3He-4He mixtures
    at zero pressure for temperatures below 250 mK and 3He concentrations
    below 8% - Equation (2)

    ========== ENTREE ==========

    [Temperature]
        La température du fluide en [K]
    [Pression]
        La pression en [Pa]
    [Concentration3He]
        La concentration de 3He dans le mélange 3He-4He sans unité

    ========== SORTIE ==========

    [EntropieMolaire]
        L'entropie molaire du mélange 3He-4He en [J/K/mol]

    ========== STATUS ==========

    Status : 

    ========== A FAIRE ==========

    Contrainte sur la pression à vérifier
    A verifier pour les prochaines valeurs de l'entropie 3He et 4He

    """

    ################## MODULES ###############################################

    from cryopy.helium import helium4
    from cryopy.helium import fermi
    from cryopy.helium import helium3

    ################## CONDITION 1 ####################################
    # helium 4 est superfluide, helium 3 assimilable à un liquide de fermi
    if temperature < TemperatureTransition(Pression, Concentration3He) or Concentration3He < ConcentrationConcentre(
            Temperature, Pression):
        return Concentration3He * fermi.molar_entropy(temperature, pressure, fraction_3he) + (
                1 - Concentration3He) * helium4.molar_entropy(temperature, pressure)

    else:
        # helium 4 est normal, helium 3 est normal
        return fraction_3he * helium3.molar_entropy(temperature, pressure, fraction_3he) + (
                1 - fraction_3he) * helium4.molar_entropy(temperature, pressure)

#%%
def tricritical_temperature(pressure):
    """
    ========== DESCRIPTION ==========

    This function return the temperature of helium 3 of the tricritical point at a given pressure

    ========== VALIDITE ==========

    <pressure> : [0 -> 22e5]

    ========== FROM ==========

    Chaudhry et al. (2010) 
        "Thermodynamic Properties of Liquid 3He-4He Mixtures Between 0–10 bar below 1.5 K"

    ========== INPUT ==========

    <pressure>
        -- float --
        The pressure of the mixture
        [Pa]

    ========== OUTPUT ==========

    <tricritical_temperature>
        -- float --
        The temperature of the mixture at the tricritical point
        []

    ========== STATUS ==========

    Status : 

    """
    ################## MODULES ################################################

    from cryopy.helium import helium7

    ################## CONDITION ##############################################

    assert 0 <= pressure <= 22e5 and pressure >= 0, 'The function helium7.tricritical_temperature is not defined for P = ' + str(pressure) + ' Pa'

    ################## INITIALISATION #########################################

    # Convert [Pa] to [Bar]
    pressure = pressure * 1e-5

    ################## FUNCTION ###############################################

    if pressure == 0:
        return 0.867
    else:
        return helium7.tricritical_temperature(0) - 0.12992576 * pressure / (pressure + 2.5967345) - 6.457263e-4 * pressure

#%%
def tricritical_fraction_3he(pressure):
    """
    ========== DESCRIPTION ==========

    This function return the fraction of helium 3 on the tri critical point at a given pressure

    ========== VALIDITE ==========

    <pressure> : [0 -> 22e5]

    ========== FROM ==========

    Chaudhry et al. (2010)
        "Thermodynamic Properties of Liquid 3He-4He Mixtures Between 0–10 bar below 1.5 K"

    ========== INPUT ==========

    <pressure>
        -- float --
        The pressure of the mixture
        [Pa]

    ========== OUTPUT ==========

    <tricritical_fraction_3he>
        -- float --
        The fraction of helium 3 at the tricritical point
        []

    ========== STATUS ==========

    Status : 2022-12-22

    """
    ################## MODULES ###############################################

    from cryopy.helium import helium7

    ################## CONDITION ##############################################

    assert 0 <= pressure <= 22e5, 'The function helium7.tricritical_fraction_3he is not defined for p = ' + str(pressure) + ' bar'

    ################## INITIALISATION #########################################

    ################## FUNCTION ###############################################

    if pressure == 0:
        return 0.674

    else:
        return helium7.tricritical_fraction_3he(0) + 0.3037124 * (helium7.tricritical_temperature(0) - helium7.tricritical_temperature(pressure)) - 4.41225e6 * (helium7.tricritical_temperature(0) - helium7.tricritical_temperature(pressure)) ** 9

#%%
def transition_temperature(pressure, fraction_3he):
    """
    ========== DESCRIPTION ==========

    This function return the transition temperature

    ========== VALIDITY ==========

    <pressure> : [0 -> 10e5]
    <fraction_3he> : [0 -> helium7.tricritical_fraction_3he(pressure)]
 
    ========== FROM ==========

    Chaudhry et al. (2010) 
        "Thermodynamic Properties of Liquid 3He-4He Mixtures Between 0–10 bar below 1.5 K"

    ========== ENTREE ==========

    <pressure>
        -- float --
        The pressure of the mixture
        [Pa]
        
    <fraction_3he>
        -- float --
        The fraction of helium 3 on the mixture
        []
        
    ========== SORTIE ==========

    <transition_temperature>
        -- float --
        The transition temperature 
        [K]

    ========== STATUS ==========

    Status : 

    """

    ################## MODULES ################################################

    import numpy as np
    from cryopy.helium import helium7

    ################## CONDITIONS #############################################

    assert 0 <= pressure <= 10e5 , 'The function helium7.transition_temperature is not defined for P = ' + str(pressure) + ' Pa'

    assert 0 <= fraction_3he <= helium7.tricritical_fraction_3he(pressure), 'The function helium7.transition_temperature is not defined for x = ' + str(fraction_3he * 100) + ' %'

    ################## INITIALISATION #########################################

    # Convert from [Pa] to [Bar]
    pressure = pressure * 1e-5

    D1 = np.poly1d([0.0009255,0.0024823,-2.620259])
    D2 = np.poly1d([0.0009397,0.0013175,-1.023726])

    ################## FUNCTION ###############################################

    return helium7.tricritical_temperature(pressure) + D1(pressure) * (fraction_3he - helium7.tricritical_fraction_3he(pressure)) + D2(pressure) * (fraction_3he - helium7.tricritical_fraction_3he(pressure)) ** 2

#%%
def fraction_3he_dilute(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the fraction of helium 3 inside the dilute phase

    ========== VALIDITY ==========

    <temperature> : [0 -> helium7.tricritical_temperature(pressure)]
    <pressure> : [0 -> 10e5]

    ========== FROM ==========

    Chaudhry et al. (2010) 
        "Thermodynamic Properties of Liquid 3He-4He Mixtures Between 0–10 bar below 1.5 K"

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the dilute phase
        [Pa]

    <pressure>
        -- float --
        The pressure of the dilute phase
        [Pa]

    ========== OUTPUT ==========

    <fraction_3he_dilute>
        -- float --
        The fraction of helium 3 inside the dilute phase
        []

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ################################################

    import numpy as np
    from cryopy.helium import helium7

    ################## CONDITIONS #############################################

    assert 0 <= temperature <= helium7.tricritical_temperature(pressure) , 'The function helium7.fraction_3he_dilute is not defined for T = ' + str(temperature) + ' K'

    assert 0 <= pressure <= 10e5 , 'The function  helium7.fraction_3he_dilute is not defined for P = ' + str(pressure) + ' Pa'

    ################## INITIALISATION #########################################

    # Convert from [Pa] to [Bar]
    pressure = pressure * 1e-5

    A0 = np.poly1d([0.0102283,-0.1269791,-0.209148])
    A1 = np.poly1d([0.0169801,-0.2165742,0.960222])
    A2 = np.poly1d([0.0092997,-0.1198491,0.549920])
    B = np.poly1d([-0.0020886,0.02291499,0.080280])
    
    ################## FUNCTION ###############################################
    
    return helium7.tricritical_fraction_3he(pressure) + A0(pressure) * (temperature - helium7.tricritical_temperature(pressure)) / ((temperature - helium7.tricritical_temperature(pressure)) - B(pressure)) + A1(pressure) * (temperature - helium7.tricritical_temperature(pressure)) + A2(pressure) * (temperature - helium7.tricritical_temperature(pressure)) ** 2

#%%
def fraction_3he_concentrated(temperature, pressure):
    """
    ========== DESCRIPTION ==========

    This function return the fraction of helium 3 inside the concentrate phase

    ========== VALIDITY ==========

    <temperature> : [0 -> helium7.tricritical_temperature(pressure)]
    <pressure> : [0 -> 10e5]

    ========== FROM ==========

    Chaudhry et al. (2010) 
        "Thermodynamic Properties of Liquid 3He-4He Mixtures Between 0–10 bar below 1.5 K"

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the concentrated phase
        [Pa]

    <pressure>
        -- float --
        The pressure of the concentrated phase
        [Pa]

    ========== OUTPUT ==========

    <fraction_3he_concentrate>
        -- float --
        The fraction of helium 3 inside the concentrated phase
        []

    ========== STATUS ==========

    Status : 

    """

    ################## MODULES ################################################

    import numpy as np
    from cryopy.helium import helium7

    ################## CONDITIONS #############################################

    assert 0 <= temperature <= helium7.tricritical_temperature(pressure) , 'The function helium7.fraction_3he_concentrate is not defined for T = ' + str(temperature) + ' K'

    assert 0 <= pressure <= 10e5 , 'The function helium7.fraction_3he_concentrated is not defined for P = ' + str(pressure) + ' Pa'

    ################## INITIALISATION #########################################

    # Convert [Pa] to [Bar]
    pressure = pressure * 1e-5
    
    C1 = np.poly1d([-0.0028598,0.0173549,-0.746805])
    C2 = np.poly1d([-0.0152076,0.1120251,-0.180743])
    C3 = np.poly1d([-0.0201411,0.1723264,0.316170])

    ################## FUNCTION ###############################################

    return helium7.tricritical_fraction_3he(pressure) + C1(pressure) * (temperature - helium7.tricritical_temperature(pressure)) + C2(pressure) * (temperature - helium7.tricritical_temperature(pressure)) ** 2 + C3(pressure) * (temperature - helium7.tricritical_temperature(pressure)) ** 3

#%%
def phase_diagram(pressure):
    """
    ========== DESCRIPTION ==========

    This function return the plot of the helium 3 / helium 4 phase diagram

    ========== VALIDITY ==========

    <pressure> : []

    ========== FROM ==========

    ========== INPUT ==========

    <pressure>
        -- float --
        The pressure 
        [Pa]

    ========== OUTPUT ==========

    <phase_diagram>
        -- plot --
        The helium 3 / helium 4 phase diagram
        []

    ========== STATUS ==========

    Status : 

    """

    ################## MODULES ################################################

    import numpy as np
    import matplotlib.pyplot as plt

    ################## CONDITIONS #############################################

    ################## INITIALISATION #########################################



    return 3


#%% Sandbox

import numpy as np
temperature = np.arange(0.1,0.4,0.001)

pressure = np.arange(0,10e5,1e4)
maximum = []
fraction_max = []
temperature_max = []

for p in pressure:
    x_concentrated = [fraction_3he_concentrated(t,p) for t in temperature]
    fraction_max.append(max(x_concentrated))
    maximum.append(np.argmax(x_concentrated))
    temperature_max.append(temperature[np.argmax(x_concentrated)])
    
    
import matplotlib.pyplot as plt

#coef = np.polyfit(pressure, temperature_max, 7)
coef = np.array([-1.17922125e-42,  2.07942046e-36,  9.87808000e-32, -3.05139959e-24, 3.23482929e-18, -1.84439626e-12,  6.73562647e-07,  1.50379312e-01])
f = np.poly1d(coef)

test_fit = [f(p) for p in pressure]


plt.figure()
plt.plot(pressure,temperature_max)
plt.plot(pressure,test_fit)
plt.legend(['data','fit'])

#%% Sandbox 2

pressure = 1e5
# Tricritical point
fraction_3He_tricritical = tricritical_fraction_3he(pressure)
temperature_tricritical = tricritical_temperature(pressure)

# Superfluid transition
fraction_3He_superfluid_transition_array = np.arange(0,fraction_3He_tricritical,0.001)
temperature_superfluid_transition_array = [transition_temperature(pressure, x) for x in fraction_3He_superfluid_transition_array]

# Concentrated phase
temperature_concentrated_array = np.arange(0.2,temperature_tricritical,0.001)
fraction_3he_concentrated_array = [fraction_3he_concentrated(t, pressure) for t in temperature_concentrated_array]

# Dilute phase
temperature_dilute_array = np.arange(0,temperature_tricritical,0.001)
fraction_3he_dilute_array = [fraction_3he_dilute(t, pressure) for t in temperature_dilute_array]

################## FUNCTION ###############################################

plt.figure()
plt.plot(fraction_3He_superfluid_transition_array,temperature_superfluid_transition_array,color='black')
plt.plot(fraction_3he_concentrated_array,temperature_concentrated_array,color='black')
plt.plot(fraction_3he_dilute_array,temperature_dilute_array,color='black')


#%% Sandbox 3

pressure = np.arange(0,18e5,1e4)
x = [tricritical_fraction_3he(p) for p in pressure]
t = [tricritical_temperature(p) for p in pressure]

plt.figure()
plt.plot(x,t)
plt.grid()
